@@ -42,6 +42,14 @@ my $rwaccessors = [
 ];
 Class::Accessor::Lite->mk_accessors( @$rwaccessors );
 
+my $EndOfEmail = Sisimai::String->EOM;
+my $RetryIndex = Sisimai::Reason->retry;
+my $RFC822Head = Sisimai::RFC5322->HEADERFIELDS('all');
+my $AddrHeader = {
+    'addresser' => $RFC822Head->{'addresser'},
+    'recipient' => $RFC822Head->{'recipient'},
+};
+
 sub new {
     # Constructor of Sisimai::Data
     # @param    [Hash] argvs    Data
@@ -50,28 +58,28 @@ sub new {
     my $argvs = { @_ };
     my $thing = {};
 
-    EMAIL_ADDRESS: {
-        # Create email address object
-        my $x0 = Sisimai::Address->parse( [ $argvs->{'addresser'} ] );
-        my $y0 = Sisimai::Address->parse( [ $argvs->{'recipient'} ] );
+    # Create email address object
+    my $x0 = Sisimai::Address->parse( [ $argvs->{'addresser'} ] );
+    my $y0 = Sisimai::Address->parse( [ $argvs->{'recipient'} ] );
+    my $v0 = undef;
+    my @v1 = ();
 
-        if( ref $x0 eq 'ARRAY' ) {
-            my $v = Sisimai::Address->new( shift @$x0 );
+    if( ref $x0 eq 'ARRAY' ) {
+        $v0 = Sisimai::Address->new( shift @$x0 );
 
-            if( ref $v eq 'Sisimai::Address' ) {
-                $thing->{'addresser'} = $v;
-                $thing->{'senderdomain'} = $v->host;
-            }
+        if( ref $v0 eq 'Sisimai::Address' ) {
+            $thing->{'addresser'} = $v0;
+            $thing->{'senderdomain'} = $v0->host;
         }
+    }
 
-        if( ref $y0 eq 'ARRAY' ) {
-            my $v = Sisimai::Address->new( shift @$y0 );
+    if( ref $y0 eq 'ARRAY' ) {
+        $v0 = Sisimai::Address->new( shift @$y0 );
 
-            if( ref $v eq 'Sisimai::Address' ) {
-                $thing->{'recipient'} = $v;
-                $thing->{'destination'} = $v->host;
-                $thing->{'alias'} = $argvs->{'alias'};
-            }
+        if( ref $v0 eq 'Sisimai::Address' ) {
+            $thing->{'recipient'} = $v0;
+            $thing->{'destination'} = $v0->host;
+            $thing->{'alias'} = $argvs->{'alias'};
         }
     }
     return undef unless ref $thing->{'recipient'} eq 'Sisimai::Address';
@@ -82,21 +90,19 @@ sub new {
                             $thing->{'recipient'}->address,
                             $argvs->{'timestamp'} );
 
-    TIMESTAMP: {
-        # Create Sisimai::Time object
-        $thing->{'timestamp'} = localtime Sisimai::Time->new( $argvs->{'timestamp'} );
-        $thing->{'timezoneoffset'} = $argvs->{'timezoneoffset'} // '+0000';
-    }
+    # Create Sisimai::Time object
+    $thing->{'timestamp'} = localtime Sisimai::Time->new( $argvs->{'timestamp'} );
+    $thing->{'timezoneoffset'} = $argvs->{'timezoneoffset'} // '+0000';
+
+    @v1 = ( 
+        'listid', 'subject', 'messageid', 'smtpagent', 'diagnosticcode',
+        'diagnostictype', 'deliverystatus', 'reason', 'lhost', 'rhost', 
+        'smtpcommand', 'feedbacktype', 'action', 'softbounce',
+    );
+    $thing->{ $_ } = $argvs->{ $_ } // '' for @v1;
+    $thing->{'replycode'} = Sisimai::RFC5321->getrc( $argvs->{'diagnosticcode'} );
+    $thing->{'softbounce'} = 1 if $thing->{'replycode'} =~ m/\A4/;
 
-    OTHER_VALUES: {
-        my @v = ( 
-            'listid', 'subject', 'messageid', 'smtpagent', 'diagnosticcode',
-            'diagnostictype', 'deliverystatus', 'reason', 'lhost', 'rhost', 
-            'smtpcommand', 'feedbacktype', 'action', 'softbounce',
-        );
-        $thing->{ $_ } = $argvs->{ $_ } // '' for @v;
-        $thing->{'replycode'} = Sisimai::RFC5321->getrc( $argvs->{'diagnosticcode'} );
-    }
     return bless( $thing, __PACKAGE__ );
 }
 
@@ -116,37 +122,32 @@ sub make {
     my $rfc822data = $messageobj->rfc822;
     my $fieldorder = { 'recipient' => [], 'addresser' => [] };
     my $objectlist = [];
-    my $endofemail = '';
     my $rxcommands = qr/\A(?:EHLO|HELO|MAIL|RCPT|DATA|QUIT)\z/;
+    my $givenorder = $argvs->{'order'} ? $argvs->{'order'} : {};
 
     return undef unless $messageobj->ds;
     return undef unless $messageobj->rfc822;
 
-    ORDER_OF_HEADERS: {
-        # Decide the order of email headers: user specified or system default.
-        my $o = exists $argvs->{'order'} ? $argvs->{'order'} : {};
-        if( ref $o eq 'HASH' && scalar keys %$o ) {
-            # If the order of headers for searching is specified, use the order
-            # for detecting an email address.
-            for my $e ( 'recipient', 'addresser' ) {
-                # The order should be "Array Reference".
-                next unless $o->{ $e };
-                next unless ref $o->{ $e } eq 'ARRAY';
-                next unless scalar @{ $o->{ $e } };
-                push @{ $fieldorder->{ $e } }, @{ $o->{ $e } };
-            }
-        }
-
+    # Decide the order of email headers: user specified or system default.
+    if( ref $givenorder eq 'HASH' && scalar keys %$givenorder ) {
+        # If the order of headers for searching is specified, use the order
+        # for detecting an email address.
         for my $e ( 'recipient', 'addresser' ) {
-            # If the order is empty, use default order.
-            if( not scalar @{ $fieldorder->{ $e } } ) {
-                # Load default order of each accessor.
-                Module::Load::load 'Sisimai::MTA';
-                $fieldorder->{ $e } = Sisimai::MTA->RFC822HEADERS( $e );
-            }
+            # The order should be "Array Reference".
+            next unless $givenorder->{ $e };
+            next unless ref $givenorder->{ $e } eq 'ARRAY';
+            next unless scalar @{ $givenorder->{ $e } };
+            push @{ $fieldorder->{ $e } }, @{ $givenorder->{ $e } };
+        }
+    }
+
+    for my $e ( 'recipient', 'addresser' ) {
+        # If the order is empty, use default order.
+        if( not scalar @{ $fieldorder->{ $e } } ) {
+            # Load default order of each accessor.
+            $fieldorder->{ $e } = $AddrHeader->{ $e };
         }
     }
-    $endofemail = Sisimai::MTA->EOM();
 
     LOOP_DELIVERY_STATUS: for my $e ( @{ $messageobj->ds } ) {
         # Create parameters for new() constructor.
@@ -206,7 +207,7 @@ sub make {
             push @datevalues, $e->{'date'} if $e->{'date'};
 
             # Date information did not exist in message/delivery-status part,...
-            for my $f ( @{ Sisimai::MTA->RFC822HEADERS('date') } ) {
+            for my $f ( @{ $RFC822Head->{'date'} } ) {
                 # Get the value of Date header or other date related header.
                 next unless $rfc822data->{ lc $f };
                 push @datevalues, $rfc822data->{ lc $f };
@@ -280,7 +281,7 @@ sub make {
 
             CHECK_DELIVERY_STATUS_VALUE: {
                 # Cleanup the value of "Diagnostic-Code:" header
-                $p->{'diagnosticcode'} =~ s/\s+$endofemail//;
+                $p->{'diagnosticcode'} =~ s/\s+$EndOfEmail//;
                 my $v = Sisimai::RFC3463->getdsn( $p->{'diagnosticcode'} );
                 if( $v =~ m/\A[45][.][1-9][.][1-9]\z/ ) {
                     # Use the DSN value in Diagnostic-Code:
@@ -295,7 +296,7 @@ sub make {
         $o = __PACKAGE__->new( %$p );
         next unless defined $o;
 
-        if( $o->reason eq '' || grep { $o->reason eq $_ } @{ Sisimai::Reason->retry } ) {
+        if( $o->reason eq '' || grep { $o->reason eq $_ } @$RetryIndex ) {
             # Decide the reason of email bounce
             if( Sisimai::Rhost->match( $o->rhost ) ) {
                 # Remote host dependent error
