@@ -3,40 +3,41 @@ use feature ':5.10';
 use strict;
 use warnings;
 use Sisimai::MTA;
+use Sisimai::RFC5322;
 
-sub description    { '' }
-sub headerlist     { [] }
 sub SMTPCOMMAND    { return Sisimai::MTA->SMTPCOMMAND    }
 sub EOM            { return Sisimai::MTA->EOM            }
 sub DELIVERYSTATUS { return Sisimai::MTA->DELIVERYSTATUS }
 sub LONGFIELDS     { return Sisimai::MTA->LONGFIELDS     }
 sub INDICATORS     { return Sisimai::MTA->INDICATORS     }
-sub RFC822HEADERS  { 
+sub RFC822HEADERS { 
+    # Grouped RFC822 headers
+    # @private
+    # @deprecated
+    # @param    [String] group  RFC822 Header group name
+    # @return   [Array,Hash]    RFC822 Header list
     my $class = shift;
-    my $argvs = shift;
-    return Sisimai::MTA->RFC822HEADERS( $argvs );
+    my $group = shift // return [ keys %{ Sisimai::RFC5322->HEADERFIELDS } ];
+    my $index = Sisimai::RFC5322->HEADERFIELDS( $group );
+    return $index;
 }
 
-sub smtpagent {
-    # Return MSP name: Call smtpagent() in each child class
-    # @return   [String] MSP name
-    my $class = shift; 
-    return shift // 'null';
-}
+sub smtpagent      { return Sisimai::MTA->smtpagent      }
+sub description    { return '' }
+sub headerlist     { return [] }
+sub pattern        { return {} }
 
 sub index {
     # MSP list
     # @return   [Array] MSP list with order
     my $class = shift;
     my $index = [
-        'US::Google', 'US::Yahoo', 'US::Aol', 'US::Outlook',
-        'US::AmazonSES', 'US::SendGrid',
-        'JP::EZweb', 'JP::KDDI', 'JP::Biglobe',
-        'US::Verizon', 'RU::MailRu', 'RU::Yandex', 'DE::GMX', 'DE::EinsUndEins',
-        'US::Zoho', 'US::Bigfoot', 'US::Facebook', 'UK::MessageLabs',
+        'US::Google', 'US::Yahoo', 'US::Aol', 'US::Outlook', 'US::AmazonSES', 
+        'US::SendGrid', 'US::Verizon', 'RU::MailRu', 'RU::Yandex', 'DE::GMX', 
+        'US::Bigfoot', 'US::Facebook', 'US::Zoho', 'DE::EinsUndEins',
+        'UK::MessageLabs', 'JP::EZweb', 'JP::KDDI', 'JP::Biglobe',
         'US::ReceivingSES',
     ];
-
     return $index;
 }
 
