@@ -4,27 +4,26 @@ use feature ':5.10';
 use strict;
 use warnings;
 
-my $RxMSP = {
+my $Re0 = {
     'from'       => qr/[<]?(?>postmaster[@]ezweb[.]ne[.]jp)[>]?/i,
     'subject'    => qr/\AMail System Error - Returned Mail\z/,
     'received'   => qr/\Afrom[ ](?:.+[.])?ezweb[.]ne[.]jp[ ]/,
     'message-id' => qr/[@].+[.]ezweb[.]ne[.]jp[>]\z/,
-    'begin'      => qr{\A(?:
+};
+my $Re1 = {
+    'begin'    => qr{\A(?:
          The[ ]user[(]s[)][ ]
         |Your[ ]message[ ]
         |Each[ ]of[ ]the[ ]following
         |[<][^ ]+[@][^ ]+[>]\z
         )
     }x,
-    'rfc822'     => [
-        # Do not rewrite this regular expressions with /x switch.
-        qr/\A[-]{50}/,
-        qr|\AContent-Type: message/rfc822\z|,
-    ],
-    'endof'      => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
+    'rfc822'   => qr#\A(?:[-]{50}|Content-Type:[ ]*message/rfc822)#,
+    'boundary' => qr/\A__SISIMAI_PSEUDO_BOUNDARY__\z/,
+    'endof'    => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
 };
 
-my $RxErr = {
+my $ReFailure = {
     #'notaccept' => [
     #    qr/The following recipients did not receive this message:/,
     #],
@@ -47,9 +46,14 @@ my $RxErr = {
     ],
 };
 
+my $Indicators = __PACKAGE__->INDICATORS;
+my $LongFields = Sisimai::RFC5322->LONGFIELDS;
+my $RFC822Head = Sisimai::RFC5322->HEADERFIELDS;
+
 sub description { 'au EZweb: http://www.au.kddi.com/mobile/' }
 sub smtpagent   { 'JP::EZweb' }
 sub headerlist  { return [ 'X-SPASIGN' ] }
+sub pattern     { return $Re0 }
 
 sub scan {
     # Detect an error from EZweb
@@ -76,57 +80,51 @@ sub scan {
     #   Received: from ezweb.ne.jp (wmflb12na02.ezweb.ne.jp [222.15.69.197])
     #   Received: from nmomta.auone-net.jp ([aaa.bbb.ccc.ddd]) by ...
     #
-    $match++ if $mhead->{'from'}     =~ $RxMSP->{'from'};
-    $match++ if $mhead->{'subject'}  =~ $RxMSP->{'subject'};
-    $match++ if $mhead->{'received'} =~ $RxMSP->{'received'};
+    $match++ if $mhead->{'from'}     =~ $Re0->{'from'};
+    $match++ if $mhead->{'subject'}  =~ $Re0->{'subject'};
+    $match++ if $mhead->{'received'} =~ $Re0->{'received'};
     if( defined $mhead->{'message-id'} ) {
-        $match++ if $mhead->{'message-id'} =~ $RxMSP->{'message-id'};
+        $match++ if $mhead->{'message-id'} =~ $Re0->{'message-id'};
     }
     return undef if $match < 2;
 
-    my $dscontents = [];    # (Ref->Array) SMTP session errors: message/delivery-status
-    my $rfc822head = undef; # (Ref->Array) Required header list in message/rfc822 part
-    my $rfc822part = '';    # (String) message/rfc822-headers part
+    require Sisimai::MIME;
+    require Sisimai::String;
+    require Sisimai::Address;
+
+    my $dscontents = []; push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
+    my @hasdivided = split( "\n", $$mbody );
     my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
+    my $rfc822part = '';    # (String) message/rfc822-headers part
     my $previousfn = '';    # (String) Previous field name
-
     my $readcursor = 0;     # (Integer) Points the current cursor position
-    my $indicators = __PACKAGE__->INDICATORS;
-
-    my $longfields = __PACKAGE__->LONGFIELDS;
-    my @stripedtxt = split( "\n", $$mbody );
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
 
     my $v = undef;
     my $p = '';
-    push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
-    $rfc822head = __PACKAGE__->RFC822HEADERS;
-
-    require Sisimai::MIME;
-    require Sisimai::String;
-    require Sisimai::RFC5322;
-    require Sisimai::Address;
 
     my $rxboundary = Sisimai::MIME->boundary( $mhead->{'content-type'}, 1 );
     my @rxmessages = ();
-    push @{ $RxMSP->{'rfc822'} }, qr|\A$rxboundary\z| if length $rxboundary;
-    map { push @rxmessages, @{ $RxErr->{ $_ } } } ( keys %$RxErr );
 
-    for my $e ( @stripedtxt ) {
-        # Read each line between $RxMSP->{'begin'} and $RxMSP->{'rfc822'}.
+    $Re1->{'boundary'} = qr|\A$rxboundary\z| if length $rxboundary;
+    map { push @rxmessages, @{ $ReFailure->{ $_ } } } ( keys %$ReFailure );
+
+    for my $e ( @hasdivided ) {
+        # Read each line between $Re1->{'begin'} and $Re1->{'rfc822'}.
         unless( $readcursor ) {
             # Beginning of the bounce message or delivery status part
-            $readcursor |= $indicators->{'deliverystatus'} if $e =~ $RxMSP->{'begin'};
+            $readcursor |= $Indicators->{'deliverystatus'} if $e =~ $Re1->{'begin'};
         }
 
-        unless( $readcursor & $indicators->{'message-rfc822'} ) {
+        unless( $readcursor & $Indicators->{'message-rfc822'} ) {
             # Beginning of the original message part
-            if( grep { $e =~ $_ } @{ $RxMSP->{'rfc822'} } ) {
-                $readcursor |= $indicators->{'message-rfc822'};
+            if( $e =~ $Re1->{'rfc822'} || $e =~ $Re1->{'boundary'} ) {
+                $readcursor |= $Indicators->{'message-rfc822'};
+                next;
             }
         }
 
-        if( $readcursor & $indicators->{'message-rfc822'} ) {
+        if( $readcursor & $Indicators->{'message-rfc822'} ) {
             # After "message/rfc822"
             if( $e =~ m/\A([-0-9A-Za-z]+?)[:][ ]*.+\z/ ) {
                 # Get required headers only
@@ -134,26 +132,26 @@ sub scan {
                 my $whs = lc $lhs;
 
                 $previousfn = '';
-                next unless grep { $whs eq lc( $_ ) } @$rfc822head;
+                next unless exists $RFC822Head->{ $whs };
 
-                $previousfn  = $lhs;
+                $previousfn  = lc $lhs;
                 $rfc822part .= $e."\n";
 
             } elsif( $e =~ m/\A[\s\t]+/ ) {
                 # Continued line from the previous line
-                next if $rfc822next->{ lc $previousfn };
-                $rfc822part .= $e."\n" if grep { $previousfn eq $_ } @$longfields;
+                next if $rfc822next->{ $previousfn };
+                $rfc822part .= $e."\n" if exists $LongFields->{ $previousfn };
 
             } else {
                 # Check the end of headers in rfc822 part
-                next unless grep { $previousfn eq $_ } @$longfields;
+                next unless exists $LongFields->{ $previousfn };
                 next if length $e;
-                $rfc822next->{ lc $previousfn } = 1;
+                $rfc822next->{ $previousfn } = 1;
             }
 
         } else {
             # Before "message/rfc822"
-            next unless $readcursor & $indicators->{'deliverystatus'};
+            next unless $readcursor & $Indicators->{'deliverystatus'};
             next unless length $e;
 
             $v = $dscontents->[ -1 ];
@@ -264,9 +262,9 @@ sub scan {
 
             } else {
                 # SMTP command is not RCPT
-                SESSION: for my $r ( keys %$RxErr ) {
+                SESSION: for my $r ( keys %$ReFailure ) {
                     # Verify each regular expression of session errors
-                    PATTERN: for my $rr ( @{ $RxErr->{ $r } } ) {
+                    PATTERN: for my $rr ( @{ $ReFailure->{ $r } } ) {
                         # Check each regular expression
                         next(PATTERN) unless $e->{'diagnosis'} =~ $rr;
                         $e->{'reason'} = $r;
