@@ -4,7 +4,7 @@ use feature ':5.10';
 use strict;
 use warnings;
 
-my $RxMSP = {
+my $Re0 = {
     'received' => qr/by .+[.]vtext[.]com /,
     'vtext.com' => {
         'from' => qr/\Apost_master[@]vtext[.]com\z/,
@@ -15,8 +15,22 @@ my $RxMSP = {
     },
 };
 
+my $Indicators = __PACKAGE__->INDICATORS;
+my $LongFields = Sisimai::RFC5322->LONGFIELDS;
+my $RFC822Head = Sisimai::RFC5322->HEADERFIELDS;
+
 sub description { 'Verizon Wireless: http://www.verizonwireless.com' }
 sub smtpagent   { 'US::Verizon' }
+sub pattern     { 
+    return {
+        'from'    => qr{(?:
+                         \Apost_master[@]vtext[.]com
+                        |[<]?sysadmin[@].+[.]vzwpix[.]com[>]?
+                        )\z
+                     }x,
+        'subject' => $Re0->{'vzwpix.com'}->{'subject'},
+    };
+}
 
 sub scan {
     # Detect an error from Verizon
@@ -38,49 +52,42 @@ sub scan {
 
     while(1) {
         # Check the value of "From" header
-        last unless grep { $_ =~ $RxMSP->{'received'} } @{ $mhead->{'received'} };
-        $vtext = 1 if $mhead->{'from'} =~ $RxMSP->{'vtext.com'}->{'from'};
-        $vtext = 0 if $mhead->{'from'} =~ $RxMSP->{'vzwpix.com'}->{'from'};
+        last unless grep { $_ =~ $Re0->{'received'} } @{ $mhead->{'received'} };
+        $vtext = 1 if $mhead->{'from'} =~ $Re0->{'vtext.com'}->{'from'};
+        $vtext = 0 if $mhead->{'from'} =~ $Re0->{'vzwpix.com'}->{'from'};
         last;
     }
     return undef unless defined $vtext;
 
-    my $dscontents = [];    # (Ref->Array) SMTP session errors: message/delivery-status
-    my $rfc822head = undef; # (Ref->Array) Required header list in message/rfc822 part
+    require Sisimai::MIME;
+    require Sisimai::Address;
+
+    my $dscontents = []; push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
+    my @hasdivided = split( "\n", $$mbody );
+    my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
     my $rfc822part = '';    # (String) message/rfc822-headers part
-    my $rfc822next = {};    # (Ref->Hash) Check flag for the end of headers in rfc822 part
     my $previousfn = '';    # (String) Previous field name
-
     my $readcursor = 0;     # (Integer) Points the current cursor position
-    my $indicators = __PACKAGE__->INDICATORS;
-
-    my $longfields = __PACKAGE__->LONGFIELDS;
-    my @stripedtxt = split( "\n", $$mbody );
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $senderaddr = '';    # (String) Sender address in the message body
     my $subjecttxt = '';    # (String) Subject of the original message
 
-    my $RxMTA      = {};    # (Ref->Hash) Delimiter patterns
-    my $RxErr      = {};    # (Ref->Hash) Error message patterns
+    my $Re1        = {};    # (Ref->Hash) Delimiter patterns
+    my $ReFailure  = {};    # (Ref->Hash) Error message patterns
     my $boundary00 = '';    # (String) Boundary string
 
     my $v = undef;
     my $p = '';
-    push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
-    $rfc822head = __PACKAGE__->RFC822HEADERS;
-
-    require Sisimai::MIME;
-    require Sisimai::Address;
 
     if( $vtext == 1 ) {
         # vtext.com
-        $RxMTA = {
+        $Re1 = {
             'begin'  => qr/\AError:\s/,
             'rfc822' => qr/\A__BOUNDARY_STRING_HERE__\z/,
             'endof'  => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
         };
 
-        $RxErr = {
+        $ReFailure = {
             'userunknown' => qr{
                 # The attempted recipient address does not exist.
                 550[ ][-][ ]Requested[ ]action[ ]not[ ]taken:[ ]no[ ]such[ ]user[ ]here
@@ -89,21 +96,27 @@ sub scan {
 
         $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
         $boundary00 = Sisimai::MIME->boundary( $mhead->{'content-type'} );
-        $RxMTA->{'rfc822'} = qr/\A[-]{2}$boundary00[-]{2}\z/ if length $boundary00;
+        $Re1->{'rfc822'} = qr/\A[-]{2}$boundary00[-]{2}\z/ if length $boundary00;
 
-        for my $e ( @stripedtxt ) {
-            # Read each line between $RxMSP->{'begin'} and $RxMSP->{'rfc822'}.
+        for my $e ( @hasdivided ) {
+            # Read each line between $Re0->{'begin'} and $Re0->{'rfc822'}.
             unless( $readcursor ) {
                 # Beginning of the bounce message or delivery status part
-                $readcursor |= $indicators->{'deliverystatus'} if $e =~ $RxMTA->{'begin'};
+                if( $e =~ $Re1->{'begin'} ) {
+                    $readcursor |= $Indicators->{'deliverystatus'};
+                    next;
+                }
             }
 
-            unless( $readcursor & $indicators->{'message-rfc822'} ) {
+            unless( $readcursor & $Indicators->{'message-rfc822'} ) {
                 # Beginning of the original message part
-                $readcursor |= $indicators->{'message-rfc822'} if $e =~ $RxMTA->{'rfc822'};
+                if( $e =~ $Re1->{'rfc822'} ) {
+                    $readcursor |= $Indicators->{'message-rfc822'};
+                    next;
+                }
             }
 
-            if( $readcursor & $indicators->{'message-rfc822'} ) {
+            if( $readcursor & $Indicators->{'message-rfc822'} ) {
                 # After "message/rfc822"
                 if( $e =~ m/\A\s\s([-0-9A-Za-z]+?)[:][ ]*.+\z/ ) {
                     # Get required headers only
@@ -111,26 +124,26 @@ sub scan {
                     my $whs = lc $lhs;
 
                     $previousfn = '';
-                    next unless grep { $whs eq lc( $_ ) } @$rfc822head;
+                    next unless exists $RFC822Head->{ $whs };
 
-                    $previousfn  = $lhs;
+                    $previousfn  = lc $lhs;
                     $rfc822part .= $e."\n";
 
                 } elsif( $e =~ m/\A[\s\t]+/ ) {
                     # Continued line from the previous line
-                    next if $rfc822next->{ lc $previousfn };
-                    $rfc822part .= $e."\n" if grep { $previousfn eq $_ } @$longfields;
+                    next if $rfc822next->{ $previousfn };
+                    $rfc822part .= $e."\n" if exists $LongFields->{ $previousfn };
 
                 } else {
                     # Check the end of headers in rfc822 part
-                    next unless grep { $previousfn eq $_ } @$longfields;
+                    next unless exists $LongFields->{ $previousfn };
                     next if length $e;
-                    $rfc822next->{ lc $previousfn } = 1;
+                    $rfc822next->{ $previousfn } = 1;
                 }
 
             } else {
                 # Before "message/rfc822"
-                next unless $readcursor & $indicators->{'deliverystatus'};
+                next unless $readcursor & $Indicators->{'deliverystatus'};
                 next unless length $e;
 
                 $v = $dscontents->[ -1 ];
@@ -176,13 +189,13 @@ sub scan {
 
     } else {
         # vzwpix.com
-        $RxMTA = {
+        $Re1 = {
             'begin'  => qr/\AMessage could not be delivered to mobile/,
             'rfc822' => qr/\A__BOUNDARY_STRING_HERE__\z/,
             'endof'  => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
         };
 
-        $RxErr = {
+        $ReFailure = {
             'userunknown' => qr{
                 No[ ]valid[ ]recipients[ ]for[ ]this[ ]MM
             }x,
@@ -190,21 +203,27 @@ sub scan {
 
         $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
         $boundary00 = Sisimai::MIME->boundary( $mhead->{'content-type'} );
-        $RxMTA->{'rfc822'} = qr/\A[-]{2}$boundary00[-]{2}\z/ if length $boundary00;
+        $Re1->{'rfc822'} = qr/\A[-]{2}$boundary00[-]{2}\z/ if length $boundary00;
 
-        for my $e ( @stripedtxt ) {
-            # Read each line between $RxMSP->{'begin'} and $RxMSP->{'rfc822'}.
+        for my $e ( @hasdivided ) {
+            # Read each line between $Re0->{'begin'} and $Re0->{'rfc822'}.
             unless( $readcursor ) {
                 # Beginning of the bounce message or delivery status part
-                $readcursor |= $indicators->{'deliverystatus'} if $e =~ $RxMTA->{'begin'};
+                if( $e =~ $Re1->{'begin'} ) {
+                    $readcursor |= $Indicators->{'deliverystatus'};
+                    next;
+                }
             }
 
-            unless( $readcursor & $indicators->{'message-rfc822'} ) {
+            unless( $readcursor & $Indicators->{'message-rfc822'} ) {
                 # Beginning of the original message part
-                $readcursor |= $indicators->{'message-rfc822'} if $e =~ $RxMTA->{'rfc822'};
+                if( $e =~ $Re1->{'rfc822'} ) {
+                    $readcursor |= $Indicators->{'message-rfc822'};
+                    next;
+                }
             }
 
-            if( $readcursor & $indicators->{'message-rfc822'} ) {
+            if( $readcursor & $Indicators->{'message-rfc822'} ) {
                 # After "message/rfc822"
                 if( $e =~ m/\A\s\s([-0-9A-Za-z]+?)[:][ ]*(.+)\z/ ) {
                     # Get required headers only
@@ -213,26 +232,26 @@ sub scan {
                     my $whs = lc $lhs;
 
                     $previousfn = '';
-                    next unless grep { $whs eq lc( $_ ) } @$rfc822head;
+                    next unless exists $RFC822Head->{ $whs };
 
-                    $previousfn  = $lhs;
+                    $previousfn  = lc $lhs;
                     $rfc822part .= $e."\n";
 
                 } elsif( $e =~ m/\A[\s\t]+/ ) {
                     # Continued line from the previous line
-                    next if $rfc822next->{ lc $previousfn };
-                    $rfc822part .= $e."\n" if grep { $previousfn eq $_ } @$longfields;
+                    next if $rfc822next->{ $previousfn };
+                    $rfc822part .= $e."\n" if exists $LongFields->{ $previousfn };
 
                 } else {
                     # Check the end of headers in rfc822 part
-                    next unless grep { $previousfn eq $_ } @$longfields;
+                    next unless exists $LongFields->{ $previousfn };
                     next if length $e;
-                    $rfc822next->{ lc $previousfn } = 1;
+                    $rfc822next->{ $previousfn } = 1;
                 }
 
             } else {
                 # Before "message/rfc822"
-                next unless $readcursor & $indicators->{'deliverystatus'};
+                next unless $readcursor & $Indicators->{'deliverystatus'};
                 next unless length $e;
 
                 $v = $dscontents->[ -1 ];
@@ -286,7 +305,6 @@ sub scan {
 
     require Sisimai::String;
     require Sisimai::RFC3463;
-    require Sisimai::RFC5322;
 
     for my $e ( @$dscontents ) {
         # Set default values if each value is empty.
@@ -300,9 +318,9 @@ sub scan {
         }
         $e->{'diagnosis'} = Sisimai::String->sweep( $e->{'diagnosis'} );
 
-        SESSION: for my $r ( keys %$RxErr ) {
+        SESSION: for my $r ( keys %$ReFailure ) {
             # Verify each regular expression of session errors
-            next unless $e->{'diagnosis'} =~ $RxErr->{ $r };
+            next unless $e->{'diagnosis'} =~ $ReFailure->{ $r };
             $e->{'reason'} = $r;
             last;
         }
