@@ -2,18 +2,7 @@ package Sisimai::MTA;
 use feature ':5.10';
 use strict;
 use warnings;
-
-sub description {
-    # @abstract Description of MTA
-    # @return   [String] Description of MTA module
-    return '';
-}
-
-sub headerlist {
-    # @abstract Header list required by each MTA module
-    # @return   [Array] Header list
-    return [];
-}
+use Sisimai::RFC5322;
 
 sub SMTPCOMMAND {
     # Detector for SMTP commands in a bounce mail message
@@ -26,10 +15,14 @@ sub SMTPCOMMAND {
         'data' => qr/\bDATA\b/,
     };
 }
-sub EOM { 
+sub EOM {
     # End of email message as a sentinel for parsing bounce messages
     # @private
+    # @deprecated
     # @return   [String] Fixed length string like a constant
+    my $class = shift;
+    my $klass = 'Sisimai::String';
+    warn sprintf( "***warning: %s->EOM deprecated method. Use %s->EOM() instead.", $class, $klass );
     return '__END_OF_EMAIL_MESSAGE__';
 }
 sub DELIVERYSTATUS {
@@ -57,47 +50,21 @@ sub DELIVERYSTATUS {
 sub LONGFIELDS {
     # Fields that might be long
     # @private
-    # @return   [Array] Long filed(email header) list
-    return [ 'To', 'From', 'Subject' ];
+    # @deprecated
+    # @return   [Hash] Long filed(email header) list
+    return [ keys %{ Sisimai::RFC5322->LONGFIELDS } ];
 }
 
-sub RFC822HEADERS {
+sub RFC822HEADERS { 
     # Grouped RFC822 headers
     # @private
-    # @param    [String] group  Header group name
-    # @return   [Array]         Header list
+    # @deprecated
+    # @param    [String] group  RFC822 Header group name
+    # @return   [Array,Hash]    RFC822 Header list
     my $class = shift;
-    my $group = shift // '';
-    my $heads = {
-        'messageid' => [ 'Message-Id' ],
-        'subject'   => [ 'Subject' ],
-        'listid'    => [ 'List-Id' ],
-        'date'      => [ 'Date', 'Posted-Date', 'Posted', 'Resent-Date', ],
-        'addresser' => [ 
-            'From', 'Return-Path', 'Reply-To', 'Errors-To', 'Reverse-Path', 
-            'X-Postfix-Sender', 'Envelope-From', 'X-Envelope-From',
-        ],
-        'recipient' => [ 
-            'To', 'Delivered-To', 'Forward-Path', 'Envelope-To',
-            'X-Envelope-To', 'Resent-To', 'Apparently-To'
-        ],
-    };
-
-    if( length $group ) {
-        # The 1st argument specified
-        return $heads->{ $group } if exists $heads->{ $group };
-
-        # Return all the values when the group name does not exist
-        return $heads;
-
-    } else {
-        # Flatten hash reference then return array reference
-        my $lists = [];
-        for my $e ( keys %$heads ) {
-            push @$lists, @{ $heads->{ $e } };
-        }
-        return $lists;
-    }
+    my $group = shift // return [ keys %{ Sisimai::RFC5322->HEADERFIELDS } ];
+    my $index = Sisimai::RFC5322->HEADERFIELDS( $group );
+    return $index;
 }
 
 sub INDICATORS {
@@ -111,6 +78,20 @@ sub INDICATORS {
     };
 }
 
+sub index {
+    # MTA list
+    # @return   [Array] MTA list with order
+    my $class = shift;
+    my $index = [
+        'Sendmail', 'Postfix', 'qmail', 'Exim', 'Courier', 'OpenSMTPD', 
+        'Exchange', 'MessagingServer', 'Domino', 'Notes', 'ApacheJames', 
+        'McAfee', 'MXLogic', 'MailFoundry', 'IMailServer', 'mFILTER', 
+        'Activehunter', 'InterScanMSS', 'SurfControl', 'MailMarshalSMTP',
+        'X1', 'X2', 'X3', 'X4', 'X5', 'V5sendmail', 
+    ];
+    return $index;
+}
+
 sub smtpagent {
     # @abstract Return MTA name: Call smtpagent() in each child class
     # @return   [String] MTA name
@@ -118,19 +99,22 @@ sub smtpagent {
     return shift // 'null';
 }
 
-sub index {
-    # MTA list
-    # @return   [Array] MTA list with order
-    my $class = shift;
-    my $index = [
-        'Sendmail', 'Postfix', 'qmail', 'OpenSMTPD', 'Exim', 'Courier',
-        'Exchange', 'MessagingServer', 'V5sendmail', 'ApacheJames', 'McAfee', 
-        'Domino', 'Notes', 'MXLogic', 'MailFoundry', 'IMailServer', 'mFILTER', 
-        'Activehunter', 'InterScanMSS', 'SurfControl', 'MailMarshalSMTP',
-        'X1', 'X2', 'X3', 'X4', 'X5',
-    ];
+sub description {
+    # @abstract Description of MTA
+    # @return   [String] Description of MTA module
+    return '';
+}
 
-    return $index;
+sub headerlist {
+    # @abstract Header list required by each MTA module
+    # @return   [Array] Header list
+    return [];
+}
+
+sub pattern { 
+    # @abstract Patterns for detecting MTA
+    # @return   [Hash] Pattern table
+    return {};
 }
 
 sub scan {
