@@ -6,19 +6,8 @@ use warnings;
 
 # http://www.courier-mta.org/courierdsn.html
 # courier/module.dsn/dsn*.txt
-my $RxMTA = {
-    #'from' => qr{Courier mail server at },
-    'begin'      => qr{(?:
-         DELAYS[ ]IN[ ]DELIVERING[ ]YOUR[ ]MESSAGE
-        |UNDELIVERABLE[ ]MAIL
-        )
-    }x,
-    'rfc822'     => qr{\AContent-Type:[ ]*(?:
-         message/rfc822
-        |text/rfc822-headers
-        )\z
-    }x,
-    'endof'      => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
+my $Re0 = {
+    'from'       => qr/Courier mail server at /,
     'subject'    => qr{(?:
          NOTICE:[ ]mail[ ]delivery[ ]status[.]
         |WARNING:[ ]delayed[ ]mail[.]
@@ -26,8 +15,21 @@ my $RxMTA = {
     }x,
     'message-id' => qr/\A[<]courier[.][0-9A-F]+[.]/,
 };
+my $Re1 = {
+    'begin'  => qr{(?:
+         DELAYS[ ]IN[ ]DELIVERING[ ]YOUR[ ]MESSAGE
+        |UNDELIVERABLE[ ]MAIL
+        )
+    }x,
+    'rfc822' => qr{\AContent-Type:[ ]*(?:
+         message/rfc822
+        |text/rfc822-headers
+        )\z
+    }x,
+    'endof'  => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
+};
 
-my $RxErr = {
+my $ReFailure = {
     'hostunknown' => qr{
         # courier/module.esmtp/esmtpclient.c:526| hard_error(del, ctf, "No such domain.");
         \ANo[ ]such[ ]domain[.]\z
@@ -39,15 +41,19 @@ my $RxErr = {
     }x,
 };
 
-my $RxTmp = {
+my $ReDelayed = {
     'networkerror' => qr{
         # courier/module.esmtp/esmtpclient.c:535| soft_error(del, ctf, "DNS lookup failed.");
         \ADNS[ ]lookup[ ]failed[.]\z
     },
 };
+my $Indicators = __PACKAGE__->INDICATORS;
+my $LongFields = Sisimai::RFC5322->LONGFIELDS;
+my $RFC822Head = Sisimai::RFC5322->HEADERFIELDS;
 
 sub description { 'Courier MTA' }
 sub smtpagent   { 'Courier' }
+sub pattern     { return $Re0 }
 
 sub scan {
     # Detect an error from Courier MTA
@@ -67,24 +73,20 @@ sub scan {
     my $mbody = shift // return undef;
     my $match = 0;
 
-    $match = 1 if $mhead->{'subject'} =~ $RxMTA->{'subject'};
+    $match ||= 1 if $mhead->{'from'}    =~ $Re0->{'from'};
+    $match ||= 1 if $mhead->{'subject'} =~ $Re0->{'subject'};
     if( defined $mhead->{'message-id'} ) {
         # Message-ID: <courier.4D025E3A.00001792@5jo.example.org>
-        $match = 1 if $mhead->{'message-id'} =~ $RxMTA->{'message-id'};
+        $match ||= 1 if $mhead->{'message-id'} =~ $Re0->{'message-id'};
     }
     return undef unless $match;
 
-    my $dscontents = [];    # (Ref->Array) SMTP session errors: message/delivery-status
-    my $rfc822head = undef; # (Ref->Array) Required header list in message/rfc822 part
-    my $rfc822part = '';    # (String) message/rfc822-headers part
+    my $dscontents = []; push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
+    my @hasdivided = split( "\n", $$mbody );
     my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
+    my $rfc822part = '';    # (String) message/rfc822-headers part
     my $previousfn = '';    # (String) Previous field name
-
     my $readcursor = 0;     # (Integer) Points the current cursor position
-    my $indicators = __PACKAGE__->INDICATORS;
-
-    my $longfields = __PACKAGE__->LONGFIELDS;
-    my @stripedtxt = split( "\n", $$mbody );
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $commandtxt = '';    # (String) SMTP Command name begin with the string '>>>'
     my $connvalues = 0;     # (Integer) Flag, 1 if all the value of $connheader have been set
@@ -96,22 +98,26 @@ sub scan {
 
     my $v = undef;
     my $p = '';
-    push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
-    $rfc822head = __PACKAGE__->RFC822HEADERS;
 
-    for my $e ( @stripedtxt ) {
-        # Read each line between $RxMTA->{'begin'} and $RxMTA->{'rfc822'}.
+    for my $e ( @hasdivided ) {
+        # Read each line between $Re1->{'begin'} and $Re1->{'rfc822'}.
         unless( $readcursor ) {
             # Beginning of the bounce message or delivery status part
-            $readcursor |= $indicators->{'deliverystatus'} if $e =~ $RxMTA->{'begin'};
+            if( $e =~ $Re1->{'begin'} ) {
+                $readcursor |= $Indicators->{'deliverystatus'};
+                next;
+            }
         }
 
-        unless( $readcursor & $indicators->{'message-rfc822'} ) {
+        unless( $readcursor & $Indicators->{'message-rfc822'} ) {
             # Beginning of the original message part
-            $readcursor |= $indicators->{'message-rfc822'} if $e =~ $RxMTA->{'rfc822'};
+            if( $e =~ $Re1->{'rfc822'} ) {
+                $readcursor |= $Indicators->{'message-rfc822'};
+                next;
+            }
         }
 
-        if( $readcursor & $indicators->{'message-rfc822'} ) {
+        if( $readcursor & $Indicators->{'message-rfc822'} ) {
             # After "message/rfc822"
             if( $e =~ m/\A([-0-9A-Za-z]+?)[:][ ]*.+\z/ ) {
                 # Get required headers only
@@ -119,26 +125,26 @@ sub scan {
                 my $whs = lc $lhs;
 
                 $previousfn = '';
-                next unless grep { $whs eq lc( $_ ) } @$rfc822head;
+                next unless exists $RFC822Head->{ $whs };
 
-                $previousfn  = $lhs;
+                $previousfn  = lc $lhs;
                 $rfc822part .= $e."\n";
 
             } elsif( $e =~ m/\A[\s\t]+/ ) {
                 # Continued line from the previous line
-                next if $rfc822next->{ lc $previousfn };
-                $rfc822part .= $e."\n" if grep { $previousfn eq $_ } @$longfields;
+                next if $rfc822next->{ $previousfn };
+                $rfc822part .= $e."\n" if exists $LongFields->{ $previousfn };
 
             } else {
                 # Check the end of headers in rfc822 part
-                next unless grep { $previousfn eq $_ } @$longfields;
+                next unless exists $LongFields->{ $previousfn };
                 next if length $e;
-                $rfc822next->{ lc $previousfn } = 1;
+                $rfc822next->{ $previousfn } = 1;
             }
 
         } else {
             # Before "message/rfc822"
-            next unless $readcursor & $indicators->{'deliverystatus'};
+            next unless $readcursor & $Indicators->{'deliverystatus'};
             next unless length $e;
 
             if( $connvalues == scalar( keys %$connheader ) ) {
@@ -254,7 +260,6 @@ sub scan {
     return undef unless $recipients;
     require Sisimai::String;
     require Sisimai::RFC3463;
-    require Sisimai::RFC5322;
 
     for my $e ( @$dscontents ) {
         # Set default values if each value is empty.
@@ -269,18 +274,18 @@ sub scan {
         }
 
         SESSION: while(1) {
-            HARD_E: for my $r ( keys %$RxErr ) {
+            HARD_E: for my $r ( keys %$ReFailure ) {
                 # Verify each regular expression of session errors
-                next unless $e->{'diagnosis'} =~ $RxErr->{ $r };
+                next unless $e->{'diagnosis'} =~ $ReFailure->{ $r };
                 $e->{'reason'} = $r;
                 $e->{'softbounce'} = 0;
                 last(HARD_E);
             }
             last if $e->{'reason'};
 
-            SOFT_E: for my $r ( keys %$RxTmp ) {
+            SOFT_E: for my $r ( keys %$ReDelayed ) {
                 # Verify each regular expression of session errors
-                next unless $e->{'diagnosis'} =~ $RxTmp->{ $r };
+                next unless $e->{'diagnosis'} =~ $ReDelayed->{ $r };
                 $e->{'reason'} = $r;
                 $e->{'softbounce'} = 1;
                 last(SOFT_E);
