@@ -4,22 +4,23 @@ use feature ':5.10';
 use strict;
 use warnings;
 
-my $RxMTA = {
-    'begin'    => qr/\AYour message/,
-    'error'    => qr/\Adid not reach the following recipient[(]s[)]:/,
-    'rfc822'   => qr|\AContent-Type: message/rfc822|,
-    'endof'    => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
+my $Re0 = {
     # X-Mailer: Internet Mail Service (5.0.1461.28)
     # X-Mailer: Microsoft Exchange Server Internet Mail Connector Version ...
-    'x-mailer' => qr{\A(?:
+    'x-mailer'  => qr{\A(?:
          Internet[ ]Mail[ ]Service[ ][(][\d.]+[)]\z
         |Microsoft[ ]Exchange[ ]Server[ ]Internet[ ]Mail[ ]Connector
         )
     }x,
-    'x-mimeole'=> qr/\AProduced By Microsoft Exchange/,
-
+    'x-mimeole' => qr/\AProduced By Microsoft Exchange/,
     # Received: by ***.**.** with Internet Mail Service (5.5.2657.72)
-    'received' => qr/\Aby .+ with Internet Mail Service [(][\d.]+[)]/,
+    'received'  => qr/\Aby .+ with Internet Mail Service [(][\d.]+[)]/,
+};
+my $Re1 = {
+    'begin'  => qr/\AYour message/,
+    'error'  => qr/\Adid not reach the following recipient[(]s[)]:/,
+    'rfc822' => qr|\AContent-Type: message/rfc822|,
+    'endof'  => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
 };
 
 my $ErrorCodeTable = {
@@ -50,9 +51,14 @@ my $ErrorCodeTable = {
     ],
 };
 
+my $Indicators = __PACKAGE__->INDICATORS;
+my $LongFields = Sisimai::RFC5322->LONGFIELDS;
+my $RFC822Head = Sisimai::RFC5322->HEADERFIELDS;
+
 sub description { 'Microsoft Exchange Server' }
 sub smtpagent   { 'Exchange' }
-sub headerlist  { return [ 'X-MS-Embedded-Report', 'X-Mailer', 'X-MimeOLE' ] };
+sub headerlist  { return [ 'X-MS-Embedded-Report', 'X-MimeOLE' ] };
+sub pattern     { return $Re0 }
 
 sub scan {
     # Detect an error from Microsoft Exchange Server
@@ -72,26 +78,28 @@ sub scan {
     my $mbody = shift // return undef;
     my $match = 0;
 
-    $match = 1 if defined $mhead->{'x-ms-embedded-report'};
+    $match ||= 1 if defined $mhead->{'x-ms-embedded-report'};
     EXCHANGE_OR_NOT: while( 1 ) {
         # Check the value of X-Mailer header
+        last if $match;
+
         if( defined $mhead->{'x-mailer'} ) {
             # X-Mailer:  Microsoft Exchange Server Internet Mail Connector Version 4.0.994.63
             # X-Mailer: Internet Mail Service (5.5.2232.9)
-            $match = 1 if $mhead->{'x-mailer'} =~ $RxMTA->{'x-mailer'};
+            $match ||= 1 if $mhead->{'x-mailer'} =~ $Re0->{'x-mailer'};
             last if $match;
         }
 
         if( defined $mhead->{'x-mimeole'} ) {
             # X-MimeOLE: Produced By Microsoft Exchange V6.5
-            $match = 1 if $mhead->{'x-mimeole'} =~ $RxMTA->{'x-mimeole'};
+            $match ||= 1 if $mhead->{'x-mimeole'} =~ $Re0->{'x-mimeole'};
             last if $match;
         }
 
         last unless scalar @{ $mhead->{'received'} };
         for my $e ( @{ $mhead->{'received'} } ) {
             # Received: by ***.**.** with Internet Mail Service (5.5.2657.72)
-            next unless $e =~ $RxMTA->{'received'};
+            next unless $e =~ $Re0->{'received'};
             $match = 1;
             last(EXCHANGE_OR_NOT);
         }
@@ -99,17 +107,12 @@ sub scan {
     }
     return undef unless $match;
 
-    my $dscontents = [];    # (Ref->Array) SMTP session errors: message/delivery-status
-    my $rfc822head = undef; # (Ref->Array) Required header list in message/rfc822 part
-    my $rfc822part = '';    # (String) message/rfc822-headers part
+    my $dscontents = []; push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
+    my @hasdivided = split( "\n", $$mbody );
     my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
+    my $rfc822part = '';    # (String) message/rfc822-headers part
     my $previousfn = '';    # (String) Previous field name
-
     my $readcursor = 0;     # (Integer) Points the current cursor position
-    my $indicators = __PACKAGE__->INDICATORS;
-
-    my $longfields = __PACKAGE__->LONGFIELDS;
-    my @stripedtxt = split( "\n", $$mbody );
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $statuspart = 0;     # (Integer) Flag, 1 = have got delivery status part.
     my $connvalues = 0;     # (Integer) Flag, 1 if all the value of $connheader have been set
@@ -121,22 +124,26 @@ sub scan {
 
     my $v = undef;
     my $p = '';
-    push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
-    $rfc822head = __PACKAGE__->RFC822HEADERS;
 
-    for my $e ( @stripedtxt ) {
-        # Read each line between $RxMTA->{'begin'} and $RxMTA->{'rfc822'}.
+    for my $e ( @hasdivided ) {
+        # Read each line between $Re1->{'begin'} and $Re1->{'rfc822'}.
         unless( $readcursor ) {
             # Beginning of the bounce message or delivery status part
-            $readcursor |= $indicators->{'deliverystatus'} if $e =~ $RxMTA->{'begin'};
+            if( $e =~ $Re1->{'begin'} ) {
+                $readcursor |= $Indicators->{'deliverystatus'};
+                next;
+            }
         }
 
-        unless( $readcursor & $indicators->{'message-rfc822'} ) {
+        unless( $readcursor & $Indicators->{'message-rfc822'} ) {
             # Beginning of the original message part
-            $readcursor |= $indicators->{'message-rfc822'} if $e =~ $RxMTA->{'rfc822'};
+            if( $e =~ $Re1->{'rfc822'} ) {
+                $readcursor |= $Indicators->{'message-rfc822'};
+                next;
+            }
         }
 
-        if( $readcursor & $indicators->{'message-rfc822'} ) {
+        if( $readcursor & $Indicators->{'message-rfc822'} ) {
             # After "message/rfc822"
             if( $e =~ m/\A([-0-9A-Za-z]+?)[:][ ]*.+\z/ ) {
                 # Get required headers only
@@ -144,26 +151,26 @@ sub scan {
                 my $whs = lc $lhs;
 
                 $previousfn = '';
-                next unless grep { $whs eq lc( $_ ) } @$rfc822head;
+                next unless exists $RFC822Head->{ $whs };
 
-                $previousfn  = $lhs;
+                $previousfn  = lc $lhs;
                 $rfc822part .= $e."\n";
 
             } elsif( $e =~ m/\A[\s\t]+/ ) {
                 # Continued line from the previous line
-                next if $rfc822next->{ lc $previousfn };
-                $rfc822part .= $e."\n" if grep { $previousfn eq $_ } @$longfields;
+                $rfc822part .= $e."\n" if exists $LongFields->{ $previousfn };
+                next if $rfc822next->{ $previousfn };
 
             } else {
                 # Check the end of headers in rfc822 part
-                next unless grep { $previousfn eq $_ } @$longfields;
+                next unless exists $LongFields->{ $previousfn };
                 next if length $e;
-                $rfc822next->{ lc $previousfn } = 1;
+                $rfc822next->{ $previousfn } = 1;
             }
 
         } else {
             # Before "message/rfc822"
-            next unless $readcursor & $indicators->{'deliverystatus'};
+            next unless $readcursor & $Indicators->{'deliverystatus'};
             next if $statuspart;
 
             if( $connvalues == scalar( keys %$connheader ) ) {
@@ -252,7 +259,6 @@ sub scan {
     return undef unless $recipients;
     require Sisimai::String;
     require Sisimai::RFC3463;
-    require Sisimai::RFC5322;
 
     for my $e ( @$dscontents ) {
 
