@@ -5,11 +5,8 @@ use strict;
 use warnings;
 
 # Based on Sisimai::MTA::Exim
-my $RxMTA = {
-    'from'    => qr/\AMail Delivery System/,
-    'rfc822'  => qr/\AIncluded is a copy of the message header:\z/,
-    'begin'   => qr/\AThis message was created automatically by mail delivery software[.]\z/,
-    'endof'   => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
+my $Re0 = {
+    'from'      => qr/\AMail Delivery System/,
     'subject'   => qr{(?:
          Mail[ ]delivery[ ]failed(:[ ]returning[ ]message[ ]to[ ]sender)?
         |Warning:[ ]message[ ].+[ ]delayed[ ]+
@@ -18,18 +15,18 @@ my $RxMTA = {
     }x,
     'message-id' => qr/\A[<]mxl[~][0-9a-f]+/,
 };
+my $Re1 = {
+    'rfc822' => qr/\AIncluded is a copy of the message header:\z/,
+    'begin'  => qr/\AThis message was created automatically by mail delivery software[.]\z/,
+    'endof'  => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
+};
 
-my $RxComm = [
+my $ReCommand = [
     qr/SMTP error from remote (?:mail server|mailer) after ([A-Za-z]{4})/,
     qr/SMTP error from remote (?:mail server|mailer) after end of ([A-Za-z]{4})/,
 ];
 
-my $RxSess = {
-    'expired' => qr{(?:
-         retry[ ]timeout[ ]exceeded
-        |No[ ]action[ ]is[ ]required[ ]on[ ]your[ ]part
-        )
-    }x,
+my $ReFailure = {
     'userunknown' => qr{
         user[ ]not[ ]found
     }x,
@@ -54,16 +51,36 @@ my $RxSess = {
     'systemerror' => qr{(?>
          delivery[ ]to[ ](?:file|pipe)[ ]forbidden
         |local[ ]delivery[ ]failed
+        |LMTP[ ]error[ ]after[ ]
         )
     }x,
     'contenterror' => qr{
-        Too[ ]many[ ]["]Received["][ ]headers[ ]
+        Too[ ]many[ ]["]Received["][ ]headers
     }x,
 };
 
+my $ReDelayed = qr{(?:
+     retry[ ]timeout[ ]exceeded
+    |No[ ]action[ ]is[ ]required[ ]on[ ]your[ ]part
+    |retry[ ]time[ ]not[ ]reached[ ]for[ ]any[ ]host[ ]after[ ]a[ ]long[ ]failure[ ]period
+    |all[ ]hosts[ ]have[ ]been[ ]failing[ ]for[ ]a[ ]long[ ]time[ ]and[ ]were[ ]last[ ]tried
+    |Delay[ ]reason:[ ]
+    |Message[ ].+[ ](?:has[ ]been[ ]frozen|was[ ]frozen[ ]on[ ]arrival[ ]by[ ])
+    )
+}x;
+
+my $Indicators = __PACKAGE__->INDICATORS;
+my $LongFields = Sisimai::RFC5322->LONGFIELDS;
+my $RFC822Head = Sisimai::RFC5322->HEADERFIELDS;
+
 sub description { 'McAfee SaaS' }
 sub smtpagent   { 'MXLogic' }
-sub headerlist  { return [ 'X-MXL-NoteHash', 'X-MXL-Hash' ] }
+
+# X-MX-Bounce: mta/src/queue/bounce
+# X-MXL-NoteHash: ffffffffffffffff-0000000000000000000000000000000000000000
+# X-MXL-Hash: 4c9d4d411993da17-bbd4212b6c887f6c23bab7db4bd87ef5edc00758
+sub headerlist  { return [ 'X-MXL-NoteHash', 'X-MXL-Hash', 'X-MX-Bounce' ] }
+sub pattern     { return $Re0 }
 
 sub scan {
     # Detect an error from MXLogic
@@ -83,44 +100,44 @@ sub scan {
     my $mbody = shift // return undef;
     my $match = 0;
 
-    $match = 1 if defined $mhead->{'x-mxl-hash'};
-    $match = 1 if defined $mhead->{'x-mxl-notehash'};
-    $match = 1 if $mhead->{'subject'} =~ $RxMTA->{'subject'};
-    $match = 1 if $mhead->{'from'}    =~ $RxMTA->{'from'};
+    $match ||= 1 if defined $mhead->{'x-mx-bounce'};
+    $match ||= 1 if defined $mhead->{'x-mxl-hash'};
+    $match ||= 1 if defined $mhead->{'x-mxl-notehash'};
+    $match ||= 1 if $mhead->{'subject'} =~ $Re0->{'subject'};
+    $match ||= 1 if $mhead->{'from'}    =~ $Re0->{'from'};
     return undef unless $match;
 
-    my $dscontents = [];    # (Ref->Array) SMTP session errors: message/delivery-status
-    my $rfc822head = undef; # (Ref->Array) Required header list in message/rfc822 part
-    my $rfc822part = '';    # (String) message/rfc822-headers part
+    my $dscontents = []; push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
+    my @hasdivided = split( "\n", $$mbody );
     my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
+    my $rfc822part = '';    # (String) message/rfc822-headers part
     my $previousfn = '';    # (String) Previous field name
-
     my $readcursor = 0;     # (Integer) Points the current cursor position
-    my $indicators = __PACKAGE__->INDICATORS;
-
-    my $longfields = __PACKAGE__->LONGFIELDS;
-    my @stripedtxt = split( "\n", $$mbody );
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $localhost0 = '';    # (String) Local MTA
 
     my $v = undef;
     my $p = '';
-    push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
-    $rfc822head = __PACKAGE__->RFC822HEADERS;
 
-    for my $e ( @stripedtxt ) {
-        # Read each line between $RxMTA->{'begin'} and $RxMTA->{'rfc822'}.
+    for my $e ( @hasdivided ) {
+        # Read each line between $Re1->{'begin'} and $Re1->{'rfc822'}.
         unless( $readcursor ) {
             # Beginning of the bounce message or delivery status part
-            $readcursor |= $indicators->{'deliverystatus'} if $e =~ $RxMTA->{'begin'};
+            if( $e =~ $Re1->{'begin'} ) {
+                $readcursor |= $Indicators->{'deliverystatus'};
+                next;
+            }
         }
 
-        unless( $readcursor & $indicators->{'message-rfc822'} ) {
+        unless( $readcursor & $Indicators->{'message-rfc822'} ) {
             # Beginning of the original message part
-            $readcursor |= $indicators->{'message-rfc822'} if $e =~ $RxMTA->{'rfc822'};
+            if( $e =~ $Re1->{'rfc822'} ) {
+                $readcursor |= $Indicators->{'message-rfc822'};
+                next;
+            }
         }
 
-        if( $readcursor & $indicators->{'message-rfc822'} ) {
+        if( $readcursor & $Indicators->{'message-rfc822'} ) {
             # After "message/rfc822"
             if( $e =~ m/\A([-0-9A-Za-z]+?)[:][ ]*.+\z/ ) {
                 # Get required headers only
@@ -128,26 +145,26 @@ sub scan {
                 my $whs = lc $lhs;
 
                 $previousfn = '';
-                next unless grep { $whs eq lc( $_ ) } @$rfc822head;
+                next unless exists $RFC822Head->{ $whs };
 
-                $previousfn  = $lhs;
+                $previousfn  = lc $lhs;
                 $rfc822part .= $e."\n";
 
             } elsif( $e =~ m/\A[\s\t]+/ ) {
                 # Continued line from the previous line
-                next if $rfc822next->{ lc $previousfn };
-                $rfc822part .= $e."\n" if grep { $previousfn eq $_ } @$longfields;
+                next if $rfc822next->{ $previousfn };
+                $rfc822part .= $e."\n" if exists $LongFields->{ $previousfn };
 
             } else {
                 # Check the end of headers in rfc822 part
-                next unless grep { $previousfn eq $_ } @$longfields;
+                next unless exists $LongFields->{ $previousfn };
                 next if length $e;
-                $rfc822next->{ lc $previousfn } = 1;
+                $rfc822next->{ $previousfn } = 1;
             }
 
         } else {
             # Before "message/rfc822"
-            next unless $readcursor & $indicators->{'deliverystatus'};
+            next unless $readcursor & $Indicators->{'deliverystatus'};
             next unless length $e;
 
             # This message was created automatically by mail delivery software.
@@ -200,7 +217,6 @@ sub scan {
 
     require Sisimai::String;
     require Sisimai::RFC3463;
-    require Sisimai::RFC5322;
 
     for my $e ( @$dscontents ) {
         # Set default values if each value is empty.
@@ -227,10 +243,10 @@ sub scan {
         }
 
         if( ! $e->{'command'} ) {
-
+            # Get the SMTP command name for the session
             COMMAND: while(1) {
                 # Get the SMTP command name for the session
-                SMTP: for my $r ( @$RxComm ) {
+                SMTP: for my $r ( @$ReCommand ) {
                     # Verify each regular expression of SMTP commands
                     next unless $e->{'diagnosis'} =~ $r;
                     $e->{'command'} = uc $1;
@@ -250,13 +266,19 @@ sub scan {
                     $e->{'reason'} = 'blocked';
 
                 } else {
-
-                    SESSION: for my $r ( keys %$RxSess ) {
-                        # Verify each regular expression of session errors
-                        next unless $e->{'diagnosis'} =~ $RxSess->{ $r };
+                    # Verify each regular expression of session errors
+                    SESSION: for my $r ( keys %$ReFailure ) {
+                        # Check each regular expression
+                        next unless $e->{'diagnosis'} =~ $ReFailure->{ $r };
                         $e->{'reason'} = $r;
                         last;
                     }
+                    last if $e->{'reason'};
+
+                    if( $e->{'diagnosis'} =~ $ReDelayed ) {
+                        # The reason "expired"
+                        $e->{'reason'} = 'expired';
+                    }
                 }
                 last;
             }
