@@ -4,17 +4,24 @@ use feature ':5.10';
 use strict;
 use warnings;
 
-my $RxMTA = {
-    'begin'    => qr/\AThis is a MIME encoded message\z/,
-    'error'    => qr/\ADelivery failed for the following reason:\z/,
-    'rfc822'   => qr|\AContent-Type: message/rfc822\z|,
-    'endof'    => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
-    'received' => qr/[(]MAILFOUNDRY[)] id /,
+my $Re0 = {
     'subject'  => qr/\AMessage delivery has failed\z/,
+    'received' => qr/[(]MAILFOUNDRY[)] id /,
 };
+my $Re1 = {
+    'begin'  => qr/\AThis is a MIME encoded message\z/,
+    'error'  => qr/\ADelivery failed for the following reason:\z/,
+    'rfc822' => qr|\AContent-Type: message/rfc822\z|,
+    'endof'  => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
+};
+
+my $Indicators = __PACKAGE__->INDICATORS;
+my $LongFields = Sisimai::RFC5322->LONGFIELDS;
+my $RFC822Head = Sisimai::RFC5322->HEADERFIELDS;
 
 sub description { 'MailFoundry' }
 sub smtpagent   { 'MailFoundry' }
+sub pattern     { return $Re0 }
 
 sub scan {
     # Detect an error from MailFoundry
@@ -33,40 +40,39 @@ sub scan {
     my $mhead = shift // return undef;
     my $mbody = shift // return undef;
 
-    return undef unless $mhead->{'subject'} =~ $RxMTA->{'subject'};
-    return undef unless grep { $_ =~ $RxMTA->{'received'} } @{ $mhead->{'received'} };
+    return undef unless $mhead->{'subject'} =~ $Re0->{'subject'};
+    return undef unless grep { $_ =~ $Re0->{'received'} } @{ $mhead->{'received'} };
 
-    my $dscontents = [];    # (Ref->Array) SMTP session errors: message/delivery-status
-    my $rfc822head = undef; # (Ref->Array) Required header list in message/rfc822 part
-    my $rfc822part = '';    # (String) message/rfc822-headers part
+    my $dscontents = []; push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
+    my @hasdivided = split( "\n", $$mbody );
     my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
+    my $rfc822part = '';    # (String) message/rfc822-headers part
     my $previousfn = '';    # (String) Previous field name
-
     my $readcursor = 0;     # (Integer) Points the current cursor position
-    my $indicators = __PACKAGE__->INDICATORS;
-
-    my $longfields = __PACKAGE__->LONGFIELDS;
-    my @stripedtxt = split( "\n", $$mbody );
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
 
     my $v = undef;
     my $p = '';
-    push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
-    $rfc822head = __PACKAGE__->RFC822HEADERS;
 
-    for my $e ( @stripedtxt ) {
-        # Read each line between $RxMTA->{'begin'} and $RxMTA->{'rfc822'}.
+    for my $e ( @hasdivided ) {
+        # Read each line between $Re1->{'begin'} and $Re1->{'rfc822'}.
         unless( $readcursor ) {
             # Beginning of the bounce message or delivery status part
-            $readcursor |= $indicators->{'deliverystatus'} if $e =~ $RxMTA->{'begin'};
+            if( $e =~ $Re1->{'begin'} ) {
+                $readcursor |= $Indicators->{'deliverystatus'};
+                next;
+            }
         }
 
-        unless( $readcursor & $indicators->{'message-rfc822'} ) {
+        unless( $readcursor & $Indicators->{'message-rfc822'} ) {
             # Beginning of the original message part
-            $readcursor |= $indicators->{'message-rfc822'} if $e =~ $RxMTA->{'rfc822'};
+            if( $e =~ $Re1->{'rfc822'} ) {
+                $readcursor |= $Indicators->{'message-rfc822'};
+                next;
+            }
         }
 
-        if( $readcursor & $indicators->{'message-rfc822'} ) {
+        if( $readcursor & $Indicators->{'message-rfc822'} ) {
             # After "message/rfc822"
             if( $e =~ m/\A([-0-9A-Za-z]+?)[:][ ]*.+\z/ ) {
                 # Get required headers only
@@ -74,26 +80,26 @@ sub scan {
                 my $whs = lc $lhs;
 
                 $previousfn = '';
-                next unless grep { $whs eq lc( $_ ) } @$rfc822head;
+                next unless exists $RFC822Head->{ $whs };
 
-                $previousfn  = $lhs;
+                $previousfn  = lc $lhs;
                 $rfc822part .= $e."\n";
 
             } elsif( $e =~ m/\A[\s\t]+/ ) {
                 # Continued line from the previous line
-                next if $rfc822next->{ lc $previousfn };
-                $rfc822part .= $e."\n" if grep { $previousfn eq $_ } @$longfields;
+                next if $rfc822next->{ $previousfn };
+                $rfc822part .= $e."\n" if exists $LongFields->{ $previousfn };
 
             } else {
                 # Check the end of headers in rfc822 part
-                next unless grep { $previousfn eq $_ } @$longfields;
+                next unless exists $LongFields->{ $previousfn };
                 next if length $e;
-                $rfc822next->{ lc $previousfn } = 1;
+                $rfc822next->{ $previousfn } = 1;
             }
 
         } else {
             # Before "message/rfc822"
-            next unless $readcursor & $indicators->{'deliverystatus'};
+            next unless $readcursor & $Indicators->{'deliverystatus'};
             next unless length $e;
 
             # Unable to deliver message to: <kijitora@example.org>
@@ -115,7 +121,7 @@ sub scan {
 
             } else {
                 # Error message
-                if( $e =~ $RxMTA->{'error'} ) {
+                if( $e =~ $Re1->{'error'} ) {
                     # Delivery failed for the following reason:
                     $v->{'diagnosis'} = $e;
 
@@ -146,7 +152,6 @@ sub scan {
     return undef unless $recipients;
     require Sisimai::String;
     require Sisimai::RFC3463;
-    require Sisimai::RFC5322;
 
     for my $e ( @$dscontents ) {
         # Set default values if each value is empty.
