@@ -4,16 +4,18 @@ use feature ':5.10';
 use strict;
 use warnings;
 
-my $RxMTA = {
+my $Re0 = {
+    'x-nai'   => qr/Modified by McAfee /,
+    'subject' => qr/\ADelivery Status\z/,
+};
+my $Re1 = {
     'begin'   => qr/[-]+ The following addresses had delivery problems [-]+\z/,
     'error'   => qr|\AContent-Type: [^ ]+/[^ ]+; name="deliveryproblems[.]txt"|,
     'rfc822'  => qr|\AContent-Type: message/rfc822\z|,
     'endof'   => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
-    'x-nai'   => qr/Modified by McAfee /,
-    'subject' => qr/\ADelivery Status\z/,
 };
 
-my $RxErr = {
+my $ReFailure = {
     'userunknown' => qr{(?:
          User[ ][(].+[@].+[)][ ]unknown[.]
         |550[ ]Unknown[ ]user[ ][^ ]+[@][^ ]+
@@ -21,9 +23,14 @@ my $RxErr = {
     }x,
 };
 
+my $Indicators = __PACKAGE__->INDICATORS;
+my $LongFields = Sisimai::RFC5322->LONGFIELDS;
+my $RFC822Head = Sisimai::RFC5322->HEADERFIELDS;
+
 sub description { 'McAfee Email Appliance' }
 sub smtpagent   { 'McAfee' }
 sub headerlist  { return [ 'X-NAI-Header' ] }
+sub pattern     { return $Re0 }
 
 sub scan {
     # Detect an error from McAfee
@@ -42,44 +49,43 @@ sub scan {
     my $mhead = shift // return undef;
     my $mbody = shift // return undef;
 
-    return undef unless $mhead->{'x-nai-header'};
-    return undef unless $mhead->{'x-nai-header'} =~ $RxMTA->{'x-nai'};
-    return undef unless $mhead->{'subject'}      =~ $RxMTA->{'subject'};
+    return undef unless defined $mhead->{'x-nai-header'};
+    return undef unless $mhead->{'x-nai-header'} =~ $Re0->{'x-nai'};
+    return undef unless $mhead->{'subject'}      =~ $Re0->{'subject'};
 
-    my $dscontents = [];    # (Ref->Array) SMTP session errors: message/delivery-status
-    my $rfc822head = undef; # (Ref->Array) Required header list in message/rfc822 part
-    my $rfc822part = '';    # (String) message/rfc822-headers part
+    require Sisimai::Address;
+
+    my $dscontents = []; push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
+    my @hasdivided = split( "\n", $$mbody );
     my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
+    my $rfc822part = '';    # (String) message/rfc822-headers part
     my $previousfn = '';    # (String) Previous field name
-
     my $readcursor = 0;     # (Integer) Points the current cursor position
-    my $indicators = __PACKAGE__->INDICATORS;
-
-    my $longfields = __PACKAGE__->LONGFIELDS;
-    my @stripedtxt = split( "\n", $$mbody );
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $diagnostic = '';    # (String) Alternative diagnostic message
 
     my $v = undef;
     my $p = '';
-    push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
-    $rfc822head = __PACKAGE__->RFC822HEADERS;
 
-    require Sisimai::Address;
-
-    for my $e ( @stripedtxt ) {
-        # Read each line between $RxMTA->{'begin'} and $RxMTA->{'rfc822'}.
+    for my $e ( @hasdivided ) {
+        # Read each line between $Re1->{'begin'} and $Re1->{'rfc822'}.
         unless( $readcursor ) {
             # Beginning of the bounce message or delivery status part
-            $readcursor |= $indicators->{'deliverystatus'} if $e =~ $RxMTA->{'begin'};
+            if( $e =~ $Re1->{'begin'} ) {
+                $readcursor |= $Indicators->{'deliverystatus'};
+                next;
+            }
         }
 
-        unless( $readcursor & $indicators->{'message-rfc822'} ) {
+        unless( $readcursor & $Indicators->{'message-rfc822'} ) {
             # Beginning of the original message part
-            $readcursor |= $indicators->{'message-rfc822'} if $e =~ $RxMTA->{'rfc822'};
+            if( $e =~ $Re1->{'rfc822'} ) {
+                $readcursor |= $Indicators->{'message-rfc822'};
+                next;
+            }
         }
 
-        if( $readcursor & $indicators->{'message-rfc822'} ) {
+        if( $readcursor & $Indicators->{'message-rfc822'} ) {
             # After "message/rfc822"
             if( $e =~ m/\A([-0-9A-Za-z]+?)[:][ ]*.+\z/ ) {
                 # Get required headers only
@@ -87,26 +93,26 @@ sub scan {
                 my $whs = lc $lhs;
 
                 $previousfn = '';
-                next unless grep { $whs eq lc( $_ ) } @$rfc822head;
+                next unless exists $RFC822Head->{ $whs };
 
-                $previousfn  = $lhs;
+                $previousfn  = lc $lhs;
                 $rfc822part .= $e."\n";
 
             } elsif( $e =~ m/\A[\s\t]+/ ) {
                 # Continued line from the previous line
-                next if $rfc822next->{ lc $previousfn };
-                $rfc822part .= $e."\n" if grep { $previousfn eq $_ } @$longfields;
+                next if $rfc822next->{ $previousfn };
+                $rfc822part .= $e."\n" if exists $LongFields->{ $previousfn };
 
             } else {
                 # Check the end of headers in rfc822 part
-                next unless grep { $previousfn eq $_ } @$longfields;
+                next unless exists $LongFields->{ $previousfn };
                 next if length $e;
-                $rfc822next->{ lc $previousfn } = 1;
+                $rfc822next->{ $previousfn } = 1;
             }
 
         } else {
             # Before "message/rfc822"
-            next unless $readcursor & $indicators->{'deliverystatus'};
+            next unless $readcursor & $Indicators->{'deliverystatus'};
             next unless length $e;
 
             # Content-Type: text/plain; name="deliveryproblems.txt"
@@ -166,7 +172,6 @@ sub scan {
 
     return undef unless $recipients;
     require Sisimai::String;
-    require Sisimai::RFC5322;
 
     for my $e ( @$dscontents ) {
         # Set default values if each value is empty.
@@ -180,9 +185,9 @@ sub scan {
         }
         $e->{'diagnosis'} = Sisimai::String->sweep( $e->{'diagnosis'} || $diagnostic );
 
-        SESSION: for my $r ( keys %$RxErr ) {
+        SESSION: for my $r ( keys %$ReFailure ) {
             # Verify each regular expression of session errors
-            next unless $e->{'diagnosis'} =~ $RxErr->{ $r };
+            next unless $e->{'diagnosis'} =~ $ReFailure->{ $r };
             $e->{'reason'} = $r;
             last;
         }
