@@ -4,18 +4,25 @@ use feature ':5.10';
 use strict;
 use warnings;
 
-my $RxMTA = {
+my $Re0 = {
     'from'    => qr/\AMail Delivery Subsystem/,
+    'subject' => qr/(?:see transcript for details\z|\AWarning: )/,
+};
+my $Re1 = {
     'begin'   => qr/\A\s+[-]+ Transcript of session follows [-]+\z/,
     'error'   => qr/\A[.]+ while talking to .+[:]\z/,
     'rfc822'  => qr{\AContent-Type:[ ]*(?:message/rfc822|text/rfc822-headers)\z},
     'endof'   => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
-    'subject' => qr/(?:see transcript for details\z|\AWarning: )/,
 };
 
+my $Indicators = __PACKAGE__->INDICATORS;
+my $LongFields = Sisimai::RFC5322->LONGFIELDS;
+my $RFC822Head = Sisimai::RFC5322->HEADERFIELDS;
+
 sub description { 'Module decription' }
 sub smtpagent   { 'Module name' }
 sub headerlist  { return [ 'X-Some-UserDefined-Header' ] }
+sub pattern     { return $Re0 }
 
 sub scan {
     # @abstract      UserDefined MTA module
@@ -41,22 +48,22 @@ sub scan {
     #   - Did not matched:          return undef;
     #
     MATCH: {
-        $match = 1 if $mhead->{'subject'} =~ $RxMTA->{'subject'};
-        $match = 1 if $mhead->{'from'}    =~ $RxMTA->{'from'};
-        $match = 1 if $mhead->{'x-some-userdefined-header'};
+        $match ||= 1 if $mhead->{'subject'} =~ $Re0->{'subject'};
+        $match ||= 1 if $mhead->{'from'}    =~ $Re0->{'from'};
+        $match ||= 1 if $mhead->{'x-some-userdefined-header'};
     }
     return undef unless $match;
 
     # 2. Parse message body($mbody) of the bounce message. See some modules in
     #    lib/Sisimai/MTA or lib/Sisimai/MSP directory to implement codes.
     #
-    my $dscontents = [];    # (Ref->Array) SMTP session errors: message/delivery-status
+    my $dscontents = []; push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
+    my @hasdivided = split( "\n", $$mbody );
+    my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
     my $rfc822part = '';    # (String) message/rfc822-headers part
-    my $rfc822head = undef; # (Ref->Array) Required header list in message/rfc822 part
-    my $recipients = 0;     # (Integer) The number of recipient addresses in the bounce message
-
-    push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
-    $rfc822head = __PACKAGE__->RFC822HEADERS;
+    my $previousfn = '';    # (String) Previous field name
+    my $readcursor = 0;     # (Integer) Points the current cursor position
+    my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
 
     # The following code is dummy to be passed "make test".
     $recipients = 1;
