@@ -4,6 +4,10 @@ use feature ':5.10';
 use strict;
 use warnings;
 
+my $Re0 = {
+    'from'    => qr/\AMail Delivery Subsystem/,
+    'subject' => qr/\AReturned mail: [A-Z]/,
+};
 # Error text regular expressions which defined in src/savemail.c
 #   savemail.c:485| (void) fflush(stdout);
 #   savemail.c:486| p = queuename(e->e_parent, 'x');
@@ -20,8 +24,7 @@ use warnings;
 #   savemail.c:497|   while (fgets(buf, sizeof buf, xfile) != NULL)
 #   savemail.c:498|       putline(buf, fp, m);
 #   savemail.c:499|   (void) fclose(xfile);
-my $RxMTA = {
-    'from'    => qr/\AMail Delivery Subsystem/,
+my $Re1 = {
     'begin'   => qr/\A\s+[-]+ Transcript of session follows [-]+\z/,
     'error'   => qr/\A[.]+ while talking to .+[:]\z/,
     'rfc822'  => qr{\A\s+-----\s(?:
@@ -30,11 +33,15 @@ my $RxMTA = {
         )\s-----
     }x,
     'endof'   => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
-    'subject' => qr/\AReturned mail: [A-Z]/,
 };
 
+my $Indicators = __PACKAGE__->INDICATORS;
+my $LongFields = Sisimai::RFC5322->LONGFIELDS;
+my $RFC822Head = Sisimai::RFC5322->HEADERFIELDS;
+
 sub description { 'Sendmail version 5' }
 sub smtpagent   { 'V5sendmail' }
+sub pattern     { return $Re0 }
 
 sub scan {
     # Detect an error from V5sendmail
@@ -53,42 +60,41 @@ sub scan {
     my $mhead = shift // return undef;
     my $mbody = shift // return undef;
 
-    return undef unless $mhead->{'subject'} =~ $RxMTA->{'subject'};
+    return undef unless $mhead->{'subject'} =~ $Re0->{'subject'};
 
-    my $dscontents = [];    # (Ref->Array) SMTP session errors: message/delivery-status
-    my $rfc822head = undef; # (Ref->Array) Required header list in message/rfc822 part
-    my $rfc822part = '';    # (String) message/rfc822-headers part
+    my $dscontents = []; push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
+    my @hasdivided = split( "\n", $$mbody );
     my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
+    my $rfc822part = '';    # (String) message/rfc822-headers part
     my $previousfn = '';    # (String) Previous field name
-
     my $readcursor = 0;     # (Integer) Points the current cursor position
-    my $indicators = __PACKAGE__->INDICATORS;
-
-    my $longfields = __PACKAGE__->LONGFIELDS;
-    my @stripedtxt = split( "\n", $$mbody );
+    my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my @responding = ();    # (Array) Responses from remote server
     my @commandset = ();    # (Array) SMTP command which is sent to remote server
-    my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $anotherset = {};    # (Ref->Hash) Another error information
 
     my $v = undef;
     my $p = '';
-    push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
-    $rfc822head = __PACKAGE__->RFC822HEADERS;
 
-    for my $e ( @stripedtxt ) {
-        # Read each line between $RxMTA->{'begin'} and $RxMTA->{'rfc822'}.
+    for my $e ( @hasdivided ) {
+        # Read each line between $Re1->{'begin'} and $Re1->{'rfc822'}.
         unless( $readcursor ) {
             # Beginning of the bounce message or delivery status part
-            $readcursor |= $indicators->{'deliverystatus'} if $e =~ $RxMTA->{'begin'};
+            if( $e =~ $Re1->{'begin'} ) {
+                $readcursor |= $Indicators->{'deliverystatus'};
+                next;
+            }
         }
 
-        unless( $readcursor & $indicators->{'message-rfc822'} ) {
+        unless( $readcursor & $Indicators->{'message-rfc822'} ) {
             # Beginning of the original message part
-            $readcursor |= $indicators->{'message-rfc822'} if $e =~ $RxMTA->{'rfc822'};
+            if( $e =~ $Re1->{'rfc822'} ) {
+                $readcursor |= $Indicators->{'message-rfc822'};
+                next;
+            }
         }
 
-        if( $readcursor & $indicators->{'message-rfc822'} ) {
+        if( $readcursor & $Indicators->{'message-rfc822'} ) {
             # After "message/rfc822"
             if( $e =~ m/\A([-0-9A-Za-z]+?)[:][ ]*.+\z/ ) {
                 # Get required headers only
@@ -96,26 +102,26 @@ sub scan {
                 my $whs = lc $lhs;
 
                 $previousfn = '';
-                next unless grep { $whs eq lc( $_ ) } @$rfc822head;
+                next unless exists $RFC822Head->{ $whs };
 
-                $previousfn  = $lhs;
+                $previousfn  = lc $lhs;
                 $rfc822part .= $e."\n";
 
             } elsif( $e =~ m/\A[\s\t]+/ ) {
                 # Continued line from the previous line
-                next if $rfc822next->{ lc $previousfn };
-                $rfc822part .= $e."\n" if grep { $previousfn eq $_ } @$longfields;
+                next if $rfc822next->{ $previousfn };
+                $rfc822part .= $e."\n" if exists $LongFields->{ $previousfn };
 
             } else {
                 # Check the end of headers in rfc822 part
-                next unless grep { $previousfn eq $_ } @$longfields;
+                next unless exists $LongFields->{ $previousfn };
                 next if length $e;
-                $rfc822next->{ lc $previousfn } = 1;
+                $rfc822next->{ $previousfn } = 1;
             }
 
         } else {
             # Before "message/rfc822"
-            next unless $readcursor & $indicators->{'deliverystatus'};
+            next unless $readcursor & $Indicators->{'deliverystatus'};
             next unless length $e;
 
             #    ----- Transcript of session follows -----
@@ -155,7 +161,7 @@ sub scan {
             } else {
                 # Detect SMTP session error or connection error
                 next if $v->{'sessionerr'};
-                if( $e =~ $RxMTA->{'error'} ) { 
+                if( $e =~ $Re1->{'error'} ) { 
                     # ----- Transcript of session follows -----
                     # ... while talking to mta.example.org.:
                     $v->{'sessionerr'} = 1;
@@ -175,7 +181,7 @@ sub scan {
         $p = $e;
         $e = '';
     }
-    return unless $readcursor & $indicators->{'message-rfc822'};
+    return unless $readcursor & $Indicators->{'message-rfc822'};
 
     unless( $recipients ) {
         # Get the recipient address from the original message
@@ -188,7 +194,6 @@ sub scan {
     return undef unless $recipients;
 
     require Sisimai::String;
-    require Sisimai::RFC5322;
 
     my $n = -1;
     for my $e ( @$dscontents ) {
