@@ -4,6 +4,10 @@ use feature ':5.10';
 use strict;
 use warnings;
 
+my $Re0 = {
+    'subject'  => qr/\Afailure notice/i,
+    'received' => qr/\A[(]qmail[ ]+\d+[ ]+invoked[ ]+(?:for[ ]+bounce|from[ ]+network)[)]/,
+};
 #  qmail-remote.c:248|    if (code >= 500) {
 #  qmail-remote.c:249|      out("h"); outhost(); out(" does not like recipient.\n");
 #  qmail-remote.c:265|  if (code >= 500) quit("D"," failed on DATA command");
@@ -12,17 +16,15 @@ use warnings;
 # Characters: K,Z,D in qmail-qmqpc.c, qmail-send.c, qmail-rspawn.c
 #  K = success, Z = temporary error, D = permanent error
 #
-my $RxMTA = {
-    'begin'    => qr/\AHi[.] This is the qmail/,
-    'rfc822'   => qr/\A--- Below this line is a copy of the message[.]\z/,
-    'error'    => qr/\ARemote host said:/,
-    'sorry'    => qr/\A[Ss]orry[,.][ ]/,
-    'subject'  => qr/\Afailure notice/,
-    'received' => qr/\A[(]qmail[ ]+\d+[ ]+invoked[ ]+(?:for[ ]+bounce|from[ ]+network)[)]/,
-    'endof'    => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
+my $Re1 = {
+    'begin'  => qr/\AHi[.] This is the qmail/,
+    'rfc822' => qr/\A--- Below this line is a copy of the message[.]\z/,
+    'error'  => qr/\ARemote host said:/,
+    'sorry'  => qr/\A[Ss]orry[,.][ ]/,
+    'endof'  => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
 };
 
-my $RxSMTP = {
+my $ReSMTP = {
     'conn'  => qr{
         # Error text regular expressions which defined in qmail-remote.c
         # qmail-remote.c:225|  if (smtpcode() != 220) quit("ZConnected to "," but greeting failed");
@@ -54,12 +56,7 @@ my $RxSMTP = {
     }x,
 };
 
-my $RxComm = qr/
-    # qmail-remote-fallback.patch
-    Sorry,[ ]no[ ]SMTP[ ]connection[ ]got[ ]far[ ]enough;[ ]most[ ]progress[ ]was[ ]([A-Z]{4})[ ]
-/x;
-
-my $RxHost = qr{(?:
+my $ReHost = qr{(?:
     # qmail-remote.c:261|  if (!flagbother) quit("DGiving up on ","");
      Giving[ ]up[ ]on[ ](.+[0-9a-zA-Z])[.]?\z
     |Connected[ ]to[ ]([-0-9a-zA-Z.]+[0-9a-zA-Z])[ ]
@@ -67,7 +64,42 @@ my $RxHost = qr{(?:
     )
 }x;
 
-my $RxSess = {
+my $ReLDAP = {
+    # qmail-ldap-1.03-20040101.patch:19817 - 19866
+    'suspend'     => qr/Mailaddress is administrative?le?y disabled/,            # 5.2.1
+    'userunknown' => qr/[Ss]orry, no mailbox here by that name/,                 # 5.1.1
+    'exceedlimit' => qr/The message exeeded the maximum size the user accepts/,  # 5.2.3
+    'systemerror' => qr{(?>
+         Automatic[ ]homedir[ ]creator[ ]crashed                    # 4.3.0
+        |Illegal[ ]value[ ]in[ ]LDAP[ ]attribute                    # 5.3.5
+        |LDAP[ ]attribute[ ]is[ ]not[ ]given[ ]but[ ]mandatory      # 5.3.5
+        |Timeout[ ]while[ ]performing[ ]search[ ]on[ ]LDAP[ ]server # 4.4.3
+        |Too[ ]many[ ]results[ ]returned[ ]but[ ]needs[ ]to[ ]be[ ]unique # 5.3.5
+        |Permanent[ ]error[ ]while[ ]executing[ ]qmail[-]forward    # 5.4.4
+        |Temporary[ ](?:
+             error[ ](?:
+                 in[ ]automatic[ ]homedir[ ]creation            # 4.3.0 or 5.3.0
+                |while[ ]executing[ ]qmail[-]forward            # 4.4.4
+                )
+            |failure[ ]in[ ]LDAP[ ]lookup                       # 4.4.3
+            )
+        |Unable[ ]to[ ](?:
+             contact[ ]LDAP[ ]server                            # 4.4.3
+            |login[ ]into[ ]LDAP[ ]server,[ ]bad[ ]credentials  # 4.4.3
+            )
+        )
+    }x,
+};
+
+# userunknown + expired
+my $ReOnHold = qr/\A[^ ]+ does not like recipient[.]\s+.+this message has been in the queue too long[.]\z/;
+
+my $ReCommand = qr/
+    # qmail-remote-fallback.patch
+    Sorry,[ ]no[ ]SMTP[ ]connection[ ]got[ ]far[ ]enough;[ ]most[ ]progress[ ]was[ ]([A-Z]{4})[ ]
+/x;
+
+my $ReFailure = {
     # 'onhold' => qr//,
     'userunknown' => qr{(?:
         # qmail-local.c:589|  strerr_die1x(100,"Sorry, no mailbox here by that name. (#5.1.1)");
@@ -107,43 +139,18 @@ my $RxSess = {
         qr/Requested action not taken: mailbox unavailable [(]not enough free space[)]/,
 };
 
-my $RxExpr = qr{
+my $ReDelayed = qr{
     # qmail-send.c:922| ... (&dline[c],"I'm not going to try again; this message has been in the queue too long.\n")) nomem();
     this[ ]message[ ]has[ ]been[ ]in[ ]the[ ]queue[ ]too[ ]long[.]\z
 }x;
 
-my $RxLDAP = {
-    # qmail-ldap-1.03-20040101.patch:19817 - 19866
-    'suspend'     => qr/Mailaddress is administrative?le?y disabled/,            # 5.2.1
-    'userunknown' => qr/[Ss]orry, no mailbox here by that name/,                 # 5.1.1
-    'exceedlimit' => qr/The message exeeded the maximum size the user accepts/,  # 5.2.3
-    'systemerror' => qr{(?>
-         Automatic[ ]homedir[ ]creator[ ]crashed                    # 4.3.0
-        |Illegal[ ]value[ ]in[ ]LDAP[ ]attribute                    # 5.3.5
-        |LDAP[ ]attribute[ ]is[ ]not[ ]given[ ]but[ ]mandatory      # 5.3.5
-        |Timeout[ ]while[ ]performing[ ]search[ ]on[ ]LDAP[ ]server # 4.4.3
-        |Too[ ]many[ ]results[ ]returned[ ]but[ ]needs[ ]to[ ]be[ ]unique # 5.3.5
-        |Permanent[ ]error[ ]while[ ]executing[ ]qmail[-]forward    # 5.4.4
-        |Temporary[ ](?:
-             error[ ](?:
-                 in[ ]automatic[ ]homedir[ ]creation            # 4.3.0 or 5.3.0
-                |while[ ]executing[ ]qmail[-]forward            # 4.4.4
-                )
-            |failure[ ]in[ ]LDAP[ ]lookup                       # 4.4.3
-            )
-        |Unable[ ]to[ ](?:
-             contact[ ]LDAP[ ]server                            # 4.4.3
-            |login[ ]into[ ]LDAP[ ]server,[ ]bad[ ]credentials  # 4.4.3
-            )
-        )
-    }x,
-};
-
-# userunknown + expired
-my $RxOnHold = qr/\A[^ ]+ does not like recipient[.]\s+.+this message has been in the queue too long[.]\z/;
+my $Indicators = __PACKAGE__->INDICATORS;
+my $LongFields = Sisimai::RFC5322->LONGFIELDS;
+my $RFC822Head = Sisimai::RFC5322->HEADERFIELDS;
 
 sub description { 'qmail' }
 sub smtpagent   { 'qmail' }
+sub pattern     { return $Re0 }
 
 sub scan {
     # Detect an error from qmail
@@ -167,39 +174,40 @@ sub scan {
     # by qmail, see http://cr.yp.to/qmail.html
     #   e.g.) Received: (qmail 12345 invoked for bounce); 29 Apr 2009 12:34:56 -0000
     #         Subject: failure notice
-    $match = 1 if lc( $mhead->{'subject'} ) =~ $RxMTA->{'subject'};
-    $match = 1 if grep { $_ =~ $RxMTA->{'received'} } @{ $mhead->{'received'} };
+    $match ||= 1 if $mhead->{'subject'} =~ $Re0->{'subject'};
+    $match ||= 1 if grep { $_ =~ $Re0->{'received'} } @{ $mhead->{'received'} };
     return undef unless $match;
 
-    my $dscontents = [];    # (Ref->Array) SMTP session errors: message/delivery-status
-    my $rfc822head = undef; # (Ref->Array) Required header list in message/rfc822 part
-    my $rfc822part = '';    # (String) message/rfc822-headers part
+    my $dscontents = []; push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
+    my @hasdivided = split( "\n", $$mbody );
     my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
+    my $rfc822part = '';    # (String) message/rfc822-headers part
     my $previousfn = '';    # (String) Previous field name
-    my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
-    my @stripedtxt = split( "\n", $$mbody );
     my $readcursor = 0;     # (Integer) Points the current cursor position
-    my $indicators = __PACKAGE__->INDICATORS;
-    my $longfields = __PACKAGE__->LONGFIELDS;
+    my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
 
     my $v = undef;
     my $p = '';
-    push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
-    $rfc822head = __PACKAGE__->RFC822HEADERS;
 
-    for my $e ( @stripedtxt ) {
-        # Read each line between $RxMTA->{'begin'} and $RxMTA->{'rfc822'}.
+    for my $e ( @hasdivided ) {
+        # Read each line between $Re1->{'begin'} and $Re1->{'rfc822'}.
         unless( $readcursor ) {
             # Beginning of the bounce message or delivery status part
-            $readcursor |= $indicators->{'deliverystatus'} if $e =~ $RxMTA->{'begin'};
+            if( $e =~ $Re1->{'begin'} ) {
+                $readcursor |= $Indicators->{'deliverystatus'};
+                next;
+            }
         }
 
-        unless( $readcursor & $indicators->{'message-rfc822'} ) {
+        unless( $readcursor & $Indicators->{'message-rfc822'} ) {
             # Beginning of the original message part
-            $readcursor |= $indicators->{'message-rfc822'} if $e =~ $RxMTA->{'rfc822'};
+            if( $e =~ $Re1->{'rfc822'} ) {
+                $readcursor |= $Indicators->{'message-rfc822'};
+                next;
+            }
         }
 
-        if( $readcursor & $indicators->{'message-rfc822'} ) {
+        if( $readcursor & $Indicators->{'message-rfc822'} ) {
             # After "message/rfc822"
             if( $e =~ m/\A([-0-9A-Za-z]+?)[:][ ]*.+\z/ ) {
                 # Get required headers only
@@ -207,26 +215,26 @@ sub scan {
                 my $whs = lc $lhs;
 
                 $previousfn = '';
-                next unless grep { $whs eq lc( $_ ) } @$rfc822head;
+                next unless exists $RFC822Head->{ $whs };
 
-                $previousfn  = $lhs;
+                $previousfn  = lc $lhs;
                 $rfc822part .= $e."\n";
 
             } elsif( $e =~ m/\A[\s\t]+/ ) {
                 # Continued line from the previous line
-                next if $rfc822next->{ lc $previousfn };
-                $rfc822part .= $e."\n" if grep { $previousfn eq $_ } @$longfields;
+                next if $rfc822next->{ $previousfn };
+                $rfc822part .= $e."\n" if exists $LongFields->{ $previousfn };
 
             } else {
                 # Check the end of headers in rfc822 part
-                next unless grep { $previousfn eq $_ } @$longfields;
+                next unless exists $LongFields->{ $previousfn };
                 next if length $e;
-                $rfc822next->{ lc $previousfn } = 1;
+                $rfc822next->{ $previousfn } = 1;
             }
 
         } else {
             # Before "message/rfc822"
-            next unless $readcursor & $indicators->{'deliverystatus'};
+            next unless $readcursor & $Indicators->{'deliverystatus'};
             next unless length $e;
 
             # <kijitora@example.jp>:
@@ -249,10 +257,10 @@ sub scan {
                 # Append error message
                 next unless length $e;
                 $v->{'diagnosis'} .= $e.' ';
-                $v->{'alterrors'}  = $e if $e =~ $RxMTA->{'error'};
+                $v->{'alterrors'}  = $e if $e =~ $Re1->{'error'};
 
                 next if $v->{'rhost'};
-                $v->{'rhost'} = $1 if $e =~ $RxHost;
+                $v->{'rhost'} = $1 if $e =~ $ReHost;
             }
         } # End of if: rfc822
 
@@ -265,7 +273,6 @@ sub scan {
     return undef unless $recipients;
     require Sisimai::String;
     require Sisimai::RFC3463;
-    require Sisimai::RFC5322;
 
     for my $e ( @$dscontents ) {
         # Set default values if each value is empty.
@@ -283,15 +290,15 @@ sub scan {
 
             COMMAND: while(1) {
                 # Get the SMTP command name for the session
-                SMTP: for my $r ( keys %$RxSMTP ) {
+                SMTP: for my $r ( keys %$ReSMTP ) {
                     # Verify each regular expression of SMTP commands
-                    next unless $e->{'diagnosis'} =~ $RxSMTP->{ $r };
+                    next unless $e->{'diagnosis'} =~ $ReSMTP->{ $r };
                     $e->{'command'} = uc $r;
                     last(COMMAND);
                 }
 
                 # Verify each regular expression of patches
-                $e->{'command'} = uc $1 if $e->{'diagnosis'} =~ $RxComm;
+                $e->{'command'} = uc $1 if $e->{'diagnosis'} =~ $ReCommand;
                 last;
             }
         }
@@ -308,36 +315,36 @@ sub scan {
 
             } else {
                 # Try to match with each error message in the table
-                if( $e->{'diagnosis'} =~ $RxOnHold ) {
+                if( $e->{'diagnosis'} =~ $ReOnHold ) {
                     # To decide the reason require pattern match with 
                     # Sisimai::Reason::* modules
                     $e->{'reason'} = 'onhold';
 
                 } else {
 
-                    SESSION: for my $r ( keys %$RxSess ) {
+                    SESSION: for my $r ( keys %$ReFailure ) {
                         # Verify each regular expression of session errors
                         if( $e->{'alterrors'} ) {
                             # Check the value of "alterrors"
-                            next unless $e->{'alterrors'} =~ $RxSess->{ $r };
+                            next unless $e->{'alterrors'} =~ $ReFailure->{ $r };
                             $e->{'reason'} = $r;
                         }
                         last(SESSION) if $e->{'reason'};
 
-                        next unless $e->{'diagnosis'} =~ $RxSess->{ $r };
+                        next unless $e->{'diagnosis'} =~ $ReFailure->{ $r };
                         $e->{'reason'} = $r;
                         last(SESSION);
                     }
 
-                    LDAP: for my $r ( keys %$RxLDAP ) {
+                    LDAP: for my $r ( keys %$ReLDAP ) {
                         # Verify each regular expression of LDAP errors
-                        next unless $e->{'diagnosis'} =~ $RxLDAP->{ $r };
+                        next unless $e->{'diagnosis'} =~ $ReLDAP->{ $r };
                         $e->{'reason'} = $r;
                         last(LDAP);
                     }
 
                     last if $e->{'reason'};
-                    $e->{'reason'} = 'expired' if $e->{'diagnosis'} =~ $RxExpr;
+                    $e->{'reason'} = 'expired' if $e->{'diagnosis'} =~ $ReDelayed;
                 }
             }
             last;
