@@ -5,8 +5,10 @@ use warnings;
 use Class::Accessor::Lite;
 use Module::Load '';
 use Sisimai::ARF;
-use Sisimai::MTA;
-use Sisimai::MSP;
+use Sisimai::Order;
+use Sisimai::String;
+use Sisimai::RFC3834;
+use Sisimai::RFC5322;
 
 my $rwaccessors = [
     'from',             # [String] UNIX From line
@@ -16,67 +18,23 @@ my $rwaccessors = [
 ];
 Class::Accessor::Lite->mk_accessors( @$rwaccessors );
 
-my $DefaultMTA = Sisimai::MTA->index;
-my $DefaultMSP = Sisimai::MSP->index;
-my @HeaderList = @{ __PACKAGE__->makeheaders };
-my $ExtModules = [];
+my $EndOfEmail = Sisimai::String->EOM;
+my $DefaultSet = Sisimai::Order->another;
+my $PatternSet = Sisimai::Order->by('subject');
+my $ExtHeaders = Sisimai::Order->headers;
+my $ToBeLoaded = [];
+my $RFC822Head = Sisimai::RFC5322->HEADERFIELDS;
+my @RFC3834Set = ( map { lc $_ } @{ Sisimai::RFC3834->headerlist } );
+my @HeaderList = ( 'from', 'to', 'date', 'subject', 'content-type', 'reply-to',
+                   'message-id', 'received', 'return-path', 'x-mailer' );
+my $MultiHeads = { 'received' => 1 };
+my $IgnoreList = { 'dkim-signature' => 1 };
+my $Indicators = { 
+    'begin' => ( 1 << 1 ),
+    'endof' => ( 1 << 2 ),
+};
 
 sub ENDOFEMAIL { '__END_OF_EMAIL_MESSAGE__' };
-
-sub makeheaders {
-    # Make email header list
-    # @return   [Array] Header list to be parsed
-    my $class = shift;
-
-    # Load email headers from each MTA,MSP module
-    my @mtaclasses = map { 'Sisimai::MTA::'.$_ } @$DefaultMTA;
-    my @mspclasses = map { 'Sisimai::MSP::'.$_ } @$DefaultMSP;
-    my $headerlist = [
-        'from', 'to', 'date', 'subject', 'content-type', 'reply-to',
-        'message-id', 'received'
-    ];
-
-    MTA_MODULES: for my $e ( @mtaclasses ) {
-        # Load MTA modules saved in lib/Sisimai/MTA directory
-        eval { Module::Load::load $e };
-        next if $@;
-
-        for my $v ( @{ $e->headerlist } ) {
-            # Get header name which required each MTA module
-            my $q = lc $v;
-            next if grep { $q eq $_ } @$headerlist;
-            push @$headerlist, $q;
-        }
-    }
-
-    MSP_MODULES: for my $e ( @mspclasses ) {
-        # Load MSP modules saved in lib/Sisimai/MSP directory
-        eval { Module::Load::load $e };
-        next if $@;
-
-        for my $v ( @{ $e->headerlist } ) {
-            # Get header name which required each MSP module
-            my $q = lc $v;
-            next if grep { $q eq $_ } @$headerlist;
-            push @$headerlist, $q;
-        }
-    }
-
-    OTHERS: for my $e ( qw|Sisimai::RFC3834| ) {
-        # Load MTA modules saved in lib/Sisimai directory
-        eval { Module::Load::load $e };
-        next if $@;
-
-        for my $v ( @{ $e->headerlist } ) {
-            # Get header name which required each MTA module
-            my $q = lc $v;
-            next if grep { $q eq $_ } @$headerlist;
-            push @$headerlist, $q;
-        }
-    }
-    return $headerlist;
-}
-
 sub new {
     # Constructor of Sisimai::Message
     # @param         [Hash] argvs       Email text data
@@ -92,7 +50,7 @@ sub new {
     my $messageobj = undef;
     my $parameters = undef;
 
-    for my $e ( 'load', 'mtalist', 'msplist' ) {
+    for my $e ( 'load', 'order' ) {
         # Order of MTA, MSP modules
         next unless exists $argvs->{ $e };
         next unless ref $argvs->{ $e } eq 'ARRAY';
@@ -124,17 +82,14 @@ sub resolve {
     my $processing = { 'from' => '', 'header' => {}, 'rfc822' => '', 'ds' => [] };
     my $methodargv = {};
     my @mtamodules = ();
-    my @mspmodules = ();
 
-    for my $e ( 'load', 'mtalist', 'msplist' ) {
+    for my $e ( 'load', 'order' ) {
         # The order of MTA modules specified by user
         next unless exists $argvs->{ $e };
         next unless ref $argvs->{ $e } eq 'ARRAY';
         next unless scalar @{ $argvs->{ $e } };
 
-        push @mtamodules, @{ $argvs->{'mtalist'} } if $e eq 'mtalist';
-        push @mspmodules, @{ $argvs->{'msplist'} } if $e eq 'msplist';
-
+        push @mtamodules, @{ $argvs->{'order'} } if $e eq 'order';
         next unless $e eq 'load';
 
         # Load user defined MTA module
@@ -145,43 +100,41 @@ sub resolve {
 
             for my $w ( @{ $v->headerlist } ) {
                 # Get header name which required user defined MTA module
-                my $q = lc $w;
-                next if grep { $q eq $_ } @HeaderList;
-                unshift @HeaderList, $q;
+                $ExtHeaders->{ lc $w }->{ $v } = 1;
             }
-            push @$ExtModules, $v;
+            push @$ToBeLoaded, $v;
         }
     }
 
     for my $e ( @mtamodules ) {
         # Append the custom order of MTA modules
-        next if grep { $e eq $_ } @$DefaultMTA;
-        unshift @$DefaultMTA, $e;
-    }
-
-    for my $e ( @mspmodules ) {
-        # Append the custom order of MTA modules
-        next if grep { $e eq $_ } @$DefaultMSP;
-        unshift @$DefaultMSP, $e;
+        next if grep { $e eq $_ } @$ToBeLoaded;
+        push @$ToBeLoaded, $e;
     }
 
     EMAIL_PROCESSING: {
         # Processes: 0(split), 1(initialize), 2(text to hash), 3(rewrite body)
-        my $endofheads = 0;     # Flag: The end of the email header
-        my $getstarted = 0;     # Flag: The beginning of the email header
-        my $first5byte = '';
+        my $readcursor = 0;
+        my @hasdivided = split( "\n", $email );
         my $mailheader = '';
         my $bodystring = '';
+        my $first5byte = '';
         my $bouncedata = undef;
         my $rfc822part = undef;
+        my $tryonfirst = [];
+
+        if( substr( $hasdivided[0], 0, 5 ) eq 'From ' ) {
+            # From MAILER-DAEMON Tue Feb 11 00:00:00 2014
+            $first5byte =  shift @hasdivided;
+            $first5byte =~ y{\r\n}{}d;
+        }
 
         # 0. Split email data to headers and a body part.
-        SPLIT_EMAIL: for my $e ( split( "\n", $email ) ) {
+        SPLIT_EMAIL: for my $e ( @hasdivided ) {
             # use split() instead of regular expression.
             $e =~ y{\r\n}{}d;
-            $first5byte ||= $e if substr( $e, 0, 5 ) eq 'From ';
 
-            if( $endofheads ) {
+            if( $readcursor & $Indicators->{'endof'} ) {
                 # The body part of the email
                 $bodystring .= $e."\n";
 
@@ -190,16 +143,15 @@ sub resolve {
                 # appeare yet.
                 if( length $e == 0 ) {
                     # Blank line, it is a boundary of headers and a body part
-                    $endofheads = 1 if $getstarted;
+                    $readcursor |= $Indicators->{'endof'} if $readcursor & $Indicators->{'begin'};
 
                 } else {
                     # The header part of the email
                     $mailheader .= $e."\n";
-                    $getstarted  = 1;
+                    $readcursor |= $Indicators->{'begin'};
                 }
             }
         }
-
         return undef unless length $mailheader;
         return undef unless length $bodystring;
 
@@ -210,23 +162,23 @@ sub resolve {
         CONVERT_HEADER: {
             # 2. Convert email headers from text to hash reference
             my $currheader = '';
-            my @multiheads = ( 'Received' );
-            my @ignorelist = ( 'DKIM-Signature' );
+            my $allheaders = {};
 
+            map { $allheaders->{ $_ } = 1 } ( @HeaderList, @RFC3834Set, keys %$ExtHeaders );
             map { $processing->{'header'}->{ $_ } = undef } @HeaderList;
-            map { $processing->{'header'}->{ lc $_ } = [] } @multiheads;
+            map { $processing->{'header'}->{ lc $_ } = [] } keys %$MultiHeads;
 
             SPLIT_HEADERS: for my $e ( split( "\n", $mailheader ) ) {
-
+                # Convert email headers to hash
                 if( $e =~ m/\A([^ ]+?)[:][ ]*(.+?)\z/ ) {
                     # split the line into a header name and a header content
                     my $lhs = $1;
                     my $rhs = $2;
 
                     $currheader = lc $lhs;
-                    next unless grep { $currheader eq $_ } @HeaderList;
+                    next unless exists $allheaders->{ $currheader };
 
-                    if( grep { $currheader eq lc $_ } @multiheads ) {
+                    if( exists $MultiHeads->{ $currheader } ) {
                         # Such as 'Received' header, there are multiple headers
                         # in a single email message.
                         $rhs =~ y/\t/ /;
@@ -234,12 +186,17 @@ sub resolve {
                         push @{ $processing->{'header'}->{ $currheader } }, $rhs;
 
                     } else {
+                        # Other headers except "Received" and so on
+                        if( $ExtHeaders->{ $currheader } ) {
+                            # MTA specific header
+                            push @$tryonfirst, keys %{ $ExtHeaders->{ $currheader } };
+                        }
                         $processing->{'header'}->{ $currheader } = $rhs;
                     }
 
                 } elsif( $e =~ m/\A[\s\t]+(.+?)\z/ ) {
                     # Ignore header?
-                    next if grep { $currheader eq lc $_ } @ignorelist;
+                    next if exists $IgnoreList->{ $currheader };
 
                     # Header line continued from the previous line
                     if( ref $processing->{'header'}->{ $currheader } eq 'ARRAY' ) {
@@ -250,12 +207,33 @@ sub resolve {
                     }
                 }
             } # End of for(SPLIT_HEADERS)
+
+            # Headers for detecting MTA/MSP module
+            unless( scalar @$tryonfirst ) {
+                # Try to match the value of "Subject" with patterns generated by
+                # Sisimai::Order->by('subject') method
+                if( length $processing->{'header'}->{'subject'} ) {
+                    # Test the value of subject header
+                    for my $e ( keys %$PatternSet ) {
+                        # Get MTA list from the subject header
+                        next unless $processing->{'header'}->{'subject'} =~ $e;
+
+                        # Matched and push MTA list
+                        push @$tryonfirst, @{ $PatternSet->{ $e } };
+                        last;
+                    }
+                }
+            }
         }
 
         REWRITE_BODY: {
             # 3. Rewrite message body for detecting the bounce reason
-            $bodystring .= Sisimai::MTA->EOM;
-            $methodargv  = { 'mail' => $processing, 'body' => \$bodystring };
+            $bodystring .= $EndOfEmail;
+            $methodargv  = { 
+                'mail' => $processing, 
+                'body' => \$bodystring, 
+                'load' => $tryonfirst,
+            };
             $bouncedata  = __PACKAGE__->rewrite( %$methodargv );
         }
 
@@ -266,7 +244,6 @@ sub resolve {
 
         REWRITE_RFC822_PART: {
             # Rewrite headers of the original message in the body part
-            require Sisimai::String;
             require Sisimai::MIME;
 
             # Convert from string to hash reference
@@ -274,7 +251,6 @@ sub resolve {
                $v =~ s/^[>]+[ ]//mg;
 
             my @rfc822text = split( "\n", $v );
-            my $rfc822head = Sisimai::MTA->RFC822HEADERS;
             my $previousfn = ''; # Previous field name
             my $borderline = '__MIME_ENCODED_BOUNDARY__';
             my $mimeborder = {};
@@ -287,7 +263,8 @@ sub resolve {
                     my $rhs = $2;
 
                     $previousfn = '';
-                    next unless grep { $lhs eq lc $_ } @$rfc822head;
+
+                    next unless exists $RFC822Head->{ $lhs };
                     $previousfn = $lhs;
                     $rfc822part->{ $previousfn } //= $rhs;
 
@@ -364,39 +341,41 @@ sub rewrite {
     # @param options mail  [String] rfc822 Original message part
     # @param options mail  [Array]  ds     Delivery status list(parsed data)
     # @param options argvs [String] body   Email message body
+    # @param options argvs [Array] load    MTA/MSP module list to load on first
     # @return              [Hash]          Parsed and structured bounce mails
     my $class = shift;
     my $argvs = { @_ };
 
     my $mesgentity = $argvs->{'mail'} || return '';
     my $bodystring = $argvs->{'body'} || return '';
+    my $tryonfirst = $argvs->{'load'} || [];
+    my $haveloaded = {};
     my $mailheader = $mesgentity->{'header'};
     my $scannedset = undef;
 
-    FALLBACK_FOR_EACH_HEADER: {
-        # Set empty string if the value is undefined
-        $mailheader->{'from'}         //= '';
-        $mailheader->{'subject'}      //= '';
-        $mailheader->{'content-type'} //= '';
-    }
+    # PRECHECK_EACH_HEADER:
+    # Set empty string if the value is undefined
+    $mailheader->{'from'}         //= '';
+    $mailheader->{'subject'}      //= '';
+    $mailheader->{'content-type'} //= '';
 
-    EXPAND_FORWARDED_MESSAGE: {
-        # Check whether or not the message is a bounce mail.
-        # Pre-Process email body if it is a forwarded bounce message.
-        # Get the original text when the subject begins from 'fwd:' or 'fw:'
-        if( $mailheader->{'subject'} =~ m/\A\s*fwd?:/i ) {
-            # Delete quoted strings, quote symbols(>)
-            $$bodystring =~ s/^[>]+[ ]//gm;
-            $$bodystring =~ s/^[>]$//gm;
-        }
+    # EXPAND_FORWARDED_MESSAGE:
+    # Check whether or not the message is a bounce mail.
+    # Pre-Process email body if it is a forwarded bounce message.
+    # Get the original text when the subject begins from 'fwd:' or 'fw:'
+    if( $mailheader->{'subject'} =~ m/\A\s*fwd?:/i ) {
+        # Delete quoted strings, quote symbols(>)
+        $$bodystring =~ s/^[>]+[ ]//gm;
+        $$bodystring =~ s/^[>]$//gm;
     }
 
     SCANNER: while(1) {
         # 1. Sisimai::ARF 
-        # 2. Sisimai::MTA::*
-        # 3. Sisimai::MSP::*
-        # 4. Sisimai::RFC3464
-        # 4. Sisimai::RFC3834
+        # 2. User-Defined Module
+        # 3. MTA Module Candidates to be tried on first
+        # 4. Sisimai::MTA::* and MSP::*
+        # 5. Sisimai::RFC3464
+        # 6. Sisimai::RFC3834
         #
         if( Sisimai::ARF->is_arf( $mailheader ) ) {
             # Feedback Loop message
@@ -404,30 +383,31 @@ sub rewrite {
             last(SCANNER) if $scannedset;
         }
 
-        EXT: for my $r ( @$ExtModules ) {
+        USER_DEFINED: for my $r ( @$ToBeLoaded ) {
             # Call user defined MTA modules
+            next if exists $haveloaded->{ $r };
             $scannedset = $r->scan( $mailheader, $bodystring );
-            last(EXT) if $scannedset;
+            $haveloaded->{ $r } = 1;
+            last(SCANNER) if $scannedset;
         }
-        last(SCANNER) if $scannedset;
 
-        MTA: for my $r ( @$DefaultMTA ) {
-            # Pre-process email headers of a bounce message in standard format.
-            # Famous MTAs, such as Sendmail, Postfix, and qmail...
-            my $v = 'Sisimai::MTA::'.$r;
-            $scannedset = $v->scan( $mailheader, $bodystring );
-            last(MTA) if $scannedset;
+        TRY_ON_FIRST: while( my $r = shift @$tryonfirst ) {
+            # Try MTA module candidates which are detected from MTA specific
+            # mail headers on first
+            next if exists $haveloaded->{ $r };
+            $scannedset = $r->scan( $mailheader, $bodystring );
+            $haveloaded->{ $r } = 1;
+            last(SCANNER) if $scannedset;
         }
-        last(SCANNER) if $scannedset;
 
-        MSP: for my $r ( @$DefaultMSP ) {
-            # Pre-process email headers of a bounce message in standard format.
-            # Famous MSP: Mail Service Providers.
-            my $v = 'Sisimai::MSP::'.$r;
-            $scannedset = $v->scan( $mailheader, $bodystring );
-            last(MSP) if $scannedset;
+        DEFAULT_LIST: for my $r ( @$DefaultSet ) {
+            # MTA/MSP modules which does not have MTA specific header and did
+            # not match with any regular expressions of Subject header.
+            next if exists $haveloaded->{ $r };
+            $scannedset = $r->scan( $mailheader, $bodystring );
+            $haveloaded->{ $r } = 1;
+            last(SCANNER) if $scannedset;
         }
-        last(SCANNER) if $scannedset;
 
         # When the all of Sisimai::MTA::* modules did not return bounce data,
         # call Sisimai::RFC3464;
