@@ -2,9 +2,26 @@ package Sisimai::RFC3464;
 use feature ':5.10';
 use strict;
 use warnings;
+use Sisimai::MTA;
+use Sisimai::RFC5322;
 
 # http://tools.ietf.org/html/rfc3464
-my $RxRFC = {
+my $Re0 = {
+    'from'        => qr/\b(?:postmaster|mailer-daemon|root)[@]/i,
+    'return-path' => qr/(?:[<][>]|mailer-daemon)/i,
+    'subject'     => qr{(?>
+                         delivery[ ](?:failed|failure|report)
+                        |failure[ ]notice
+                        |mail[ ](?:delivery|error)
+                        |non[-]delivery
+                        |returned[ ]mail
+                        |undeliverable[ ]mail
+                        |Warning:[ ]
+                        )
+                     }xi,
+};
+
+my $Re1 = {
     'begin'  => qr{\A(?>
          Content-Type:[ ]*(?:
               message/delivery-status
@@ -22,30 +39,17 @@ my $RxRFC = {
         |Return-Path:[ ]*[<].+[>]\z
         )\z
     }xi,
-    'error'  => qr{\A(?:
-         [45]\d\d\s+
-        |[<][^@]+[@][^@]+[>]:?\s+
-        )
-    }xi,
+    'error'  => qr/\A(?:[45]\d\d\s+|[<][^@]+[@][^@]+[>]:?\s+)/i,
+    'command'=> qr/[ ](RCPT|MAIL|DATA)[ ]+command\b/,
 };
 
-# Regular expressions for BODY_PARSER_FOR_FALLBACK block.
-my $RxCmd = qr{[ ](RCPT|MAIL|DATA)[ ]+command\b};            # SMTP Command
-my $RxEMF = qr{\b(?:postmaster|mailer-daemon|root)[@]}i;    # From:
-my $RxEMR = qr{(?:[<][>]|mailer-daemon)}i;                  # Return-Path:
-my $RxEMS = qr{(?>                                          # Subject:
-     delivery[ ](?:failed|failure|report)
-    |failure[ ]notice
-    |mail[ ](?:delivery|error)
-    |non[-]delivery
-    |returned[ ]mail
-    |undeliverable[ ]mail
-    |Warning:[ ]
-    )
-}xi;
+my $Indicators = Sisimai::MTA->INDICATORS;
+my $LongFields = Sisimai::RFC5322->LONGFIELDS;
+my $RFC822Head = Sisimai::RFC5322->HEADERFIELDS;
 
 sub description { 'Fallback Module for MTAs' };
 sub smtpagent   { 'RFC3464' };
+sub pattern     { return $Re0 }
 
 sub scan {
     # Detect an error for RFC3464
@@ -67,23 +71,16 @@ sub scan {
     return undef unless keys %$mhead;
     return undef unless ref $mbody eq 'SCALAR';
 
-    require Sisimai::MTA;
     require Sisimai::MDA;
     require Sisimai::Address;
-    require Sisimai::RFC5322;
 
-    my $dscontents = [];    # (Ref->Array) SMTP session errors: message/delivery-status
-    my $rfc822head = undef; # (Ref->Array) Required header list in message/rfc822 part
-    my $rfc822part = '';    # (String) message/rfc822-headers part
-    my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
-    my $previousfn = '';    # (String) Previous field name
-
-    my $readcursor = 0;     # (Integer) Points the current cursor position
-    my $indicators = Sisimai::MTA->INDICATORS;
-
-    my $longfields = Sisimai::MTA->LONGFIELDS;
+    my $dscontents = []; push @$dscontents, Sisimai::MTA->DELIVERYSTATUS;
+    my @hasdivided = split( "\n", $$mbody );
     my $scannedset = Sisimai::MDA->scan( $mhead, $mbody );
-    my @stripedtxt = split( "\n", $$mbody );
+    my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
+    my $rfc822part = '';    # (String) message/rfc822-headers part
+    my $previousfn = '';    # (String) Previous field name
+    my $readcursor = 0;     # (Integer) Points the current cursor position
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $connheader = {
         'date'    => '',    # The value of Arrival-Date header
@@ -93,22 +90,26 @@ sub scan {
 
     my $v = undef;
     my $p = '';
-    push @$dscontents, Sisimai::MTA->DELIVERYSTATUS;
-    $rfc822head = Sisimai::MTA->RFC822HEADERS;
 
-    for my $e ( @stripedtxt ) {
-        # Read each line between $RxRFC->{'begin'} and $RxRFC->{'rfc822'}.
+    for my $e ( @hasdivided ) {
+        # Read each line between $Re1->{'begin'} and $Re1->{'rfc822'}.
         unless( $readcursor ) {
             # Beginning of the bounce message or delivery status part
-            $readcursor |= $indicators->{'deliverystatus'} if $e =~ $RxRFC->{'begin'};
+            if( $e =~ $Re1->{'begin'} ) {
+                $readcursor |= $Indicators->{'deliverystatus'};
+                next;
+            }
         }
 
-        unless( $readcursor & $indicators->{'message-rfc822'} ) {
+        unless( $readcursor & $Indicators->{'message-rfc822'} ) {
             # Beginning of the original message part
-            $readcursor |= $indicators->{'message-rfc822'} if $e =~ $RxRFC->{'rfc822'};
+            if( $e =~ $Re1->{'rfc822'} ) {
+                $readcursor |= $Indicators->{'message-rfc822'};
+                next;
+            }
         }
 
-        if( $readcursor & $indicators->{'message-rfc822'} ) {
+        if( $readcursor & $Indicators->{'message-rfc822'} ) {
             # After "message/rfc822"
             if( $e =~ m/\A([-0-9A-Za-z]+?)[:][ ]*(.+)\z/ ) {
                 # Get required headers only
@@ -117,26 +118,26 @@ sub scan {
                 my $whs = lc $lhs;
 
                 $previousfn = '';
-                next unless grep { $whs eq lc( $_ ) } @$rfc822head;
+                next unless exists $RFC822Head->{ $whs };
 
-                $previousfn  = $lhs;
+                $previousfn  = lc $lhs;
                 $rfc822part .= $e."\n";
 
             } elsif( $e =~ m/\A[\s\t]+/ ) {
                 # Continued line from the previous line
-                next if $rfc822next->{ lc $previousfn };
-                $rfc822part .= $e."\n" if grep { $previousfn eq $_ } @$longfields;
+                next if $rfc822next->{ $previousfn };
+                $rfc822part .= $e."\n" if exists $LongFields->{ $previousfn };
 
             } else {
                 # Check the end of headers in rfc822 part
-                next unless grep { $previousfn eq $_ } @$longfields;
+                next unless exists $LongFields->{ $previousfn };
                 next if length $e;
-                $rfc822next->{ lc $previousfn } = 1;
+                $rfc822next->{ $previousfn } = 1;
             }
 
         } else {
             # Before "message/rfc822"
-            next unless $readcursor & $indicators->{'deliverystatus'};
+            next unless $readcursor & $Indicators->{'deliverystatus'};
             next unless length $e;
 
             $v = $dscontents->[ -1 ];
@@ -320,7 +321,7 @@ sub scan {
                     } else {
                         # Get error message
                         next if $e =~ m/\A[ -]+/;
-                        next unless $e =~ $RxRFC->{'error'};
+                        next unless $e =~ $Re1->{'error'};
 
                         # 500 User Unknown
                         # <kijitora@example.jp> Unknown
@@ -340,11 +341,11 @@ sub scan {
         # Fallback, parse entire message body
         unless( $recipients ) {
             # Failed to get a recipient address at code above
-            $match = 1 if $mhead->{'from'}        =~ $RxEMF;
-            $match = 1 if $mhead->{'subject'}     =~ $RxEMS;
+            $match ||= 1 if $mhead->{'from'}    =~ $Re0->{'from'};
+            $match ||= 1 if $mhead->{'subject'} =~ $Re0->{'subject'};
             if( defined $mhead->{'return-path'} ) {
                 # Check the value of Return-Path of the message
-                $match = 1 if $mhead->{'return-path'} =~ $RxEMR;
+                $match ||= 1 if $mhead->{'return-path'} =~ $Re0->{'return-path'};
             }
             last unless $match;
 
@@ -415,8 +416,8 @@ sub scan {
             my $b = $dscontents->[ -1 ];
             for my $e ( split( "\n", $$mbody ) ) {
                 # Get the recipient's email address and error messages.
-                last if $e =~ $RxRFC->{'endof'};
-                last if $e =~ $RxRFC->{'rfc822'};
+                last if $e =~ $Re1->{'endof'};
+                last if $e =~ $Re1->{'rfc822'};
                 last if $e =~ $RxEnd;
 
                 next unless length $e;
@@ -478,7 +479,7 @@ sub scan {
             $e->{'agent'} = __PACKAGE__->smtpagent;
         }
         $e->{'status'} ||= Sisimai::RFC3463->getdsn( $e->{'diagnosis'} );
-        $e->{'command'}  = $1 if $e->{'diagnosis'} =~ $RxCmd;
+        $e->{'command'}  = $1 if $e->{'diagnosis'} =~ $Re1->{'command'};
 
         if( scalar @{ $mhead->{'received'} } ) {
             # Get localhost and remote host name from Received header.
