@@ -4,7 +4,7 @@ use strict;
 use warnings;
 
 # Regular expression of valid RFC-5322 email address(<addr-spec>)
-my $Rx = { 'rfc5322' => undef, 'ignored' => undef, 'domain' => undef, };
+my $Re = { 'rfc5322' => undef, 'ignored' => undef, 'domain' => undef, };
 
 BUILD_REGULAR_EXPRESSIONS: {
     # See http://www.ietf.org/rfc/rfc5322.txt
@@ -25,9 +25,49 @@ BUILD_REGULAR_EXPRESSIONS: {
     my $local_part     = qr/(?:$dot_atom|$quoted_string)/o;
     my $domain         = qr/(?:$dot_atom|$domain_literal)/o;
 
-    $Rx->{'rfc5322'} = qr/$local_part[@]$domain/o;
-    $Rx->{'ignored'} = qr/$local_part[.]*[@]$domain/o;
-    $Rx->{'domain'}  = qr/$domain/o;
+    $Re->{'rfc5322'} = qr/$local_part[@]$domain/o;
+    $Re->{'ignored'} = qr/$local_part[.]*[@]$domain/o;
+    $Re->{'domain'}  = qr/$domain/o;
+}
+
+my $HEADERINDEX = {};
+my $HEADERTABLE = {
+    'messageid' => [ 'Message-Id' ],
+    'subject'   => [ 'Subject' ],
+    'listid'    => [ 'List-Id' ],
+    'date'      => [ 'Date', 'Posted-Date', 'Posted', 'Resent-Date', ],
+    'addresser' => [ 
+        'From', 'Return-Path', 'Reply-To', 'Errors-To', 'Reverse-Path', 
+        'X-Postfix-Sender', 'Envelope-From', 'X-Envelope-From',
+    ],
+    'recipient' => [ 
+        'To', 'Delivered-To', 'Forward-Path', 'Envelope-To',
+        'X-Envelope-To', 'Resent-To', 'Apparently-To'
+    ],
+};
+
+BUILD_FLATTEN_RFC822HEADER_LIST: {
+    # Convert $HEADER: hash reference to flatten hash reference for being
+    # called from Sisimai::MTA::*
+    for my $v ( values %$HEADERTABLE ) {
+        $HEADERINDEX->{ lc $_ } = 1 for @$v;
+    }
+}
+
+sub HEADERFIELDS {
+    # Grouped RFC822 headers
+    # @param    [String] group  RFC822 Header group name
+    # @return   [Array,Hash]    RFC822 Header list
+    my $class = shift;
+    my $group = shift || return $HEADERINDEX;
+    return $HEADERTABLE->{ $group } if exists $HEADERTABLE->{ $group };
+    return $HEADERTABLE;
+}
+
+sub LONGFIELDS {
+    # Fields that might be long
+    # @return   [Hash] Long filed(email header) list
+    return { 'to' => 1, 'from' => 1, 'subject' => 1 };
 }
 
 sub is_emailaddress {
@@ -39,7 +79,7 @@ sub is_emailaddress {
     my $email = shift // return 0;
 
     return 0 if $email =~ m/(?:[\x00-\x1f]|\x1f)/;
-    return 1 if $email =~ $Rx->{'ignored'};
+    return 1 if $email =~ $Re->{'ignored'};
     return 0;
 }
 
@@ -53,7 +93,7 @@ sub is_domainpart {
 
     return 0 if $dpart =~ m/(?:[\x00-\x1f]|\x1f)/;
     return 0 if $dpart =~ m/[@]/;
-    return 1 if $dpart =~ $Rx->{'domain'};
+    return 1 if $dpart =~ $Re->{'domain'};
     return 0;
 }
 
